# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class OctalValuesTestCase(RuleTestCase):
    rule_id = 'octal-values'

    def test_disabled(self):
        conf = ('octal-values: disable\n'
                'new-line-at-end-of-file: disable\n'
                'document-start: disable\n')
        self.check('user-city: 010', conf)
        self.check('user-city: 0o10', conf)

    def test_implicit_octal_values(self):
        conf = ('octal-values: {forbid-implicit-octal: true}\n'
                'new-line-at-end-of-file: disable\n'
                'document-start: disable\n')
        self.check('user-city: 010', conf, problem=(1, 15))
        self.check('user-city: abc', conf)
        self.check('user-city: 010,0571', conf)
        self.check("user-city: '010'", conf)
        self.check('user-city: "010"', conf)
        self.check('user-city:\n'
                   '  - 010', conf, problem=(2, 8))
        self.check('user-city: [010]', conf, problem=(1, 16))
        self.check('user-city: {beijing: 010}', conf, problem=(1, 25))
        self.check('explicit-octal: 0o10', conf)
        self.check('not-number: 0abc', conf)
        self.check('zero: 0', conf)
        self.check('hex-value: 0x10', conf)
        self.check('number-values:\n'
                   '  - 0.10\n'
                   '  - .01\n'
                   '  - 0e3\n', conf)

    def test_explicit_octal_values(self):
        conf = ('octal-values: {forbid-explicit-octal: true}\n'
                'new-line-at-end-of-file: disable\n'
                'document-start: disable\n')
        self.check('user-city: 0o10', conf, problem=(1, 16))
        self.check('user-city: abc', conf)
        self.check('user-city: 0o10,0571', conf)
        self.check("user-city: '0o10'", conf)
        self.check('user-city:\n'
                   '  - 0o10', conf, problem=(2, 9))
        self.check('user-city: [0o10]', conf, problem=(1, 17))
        self.check('user-city: {beijing: 0o10}', conf, problem=(1, 26))
        self.check('implicit-octal: 010', conf)
        self.check('not-number: 0oabc', conf)
        self.check('zero: 0', conf)
        self.check('hex-value: 0x10', conf)
        self.check('number-values:\n'
                   '  - 0.10\n'
                   '  - .01\n'
                   '  - 0e3\n', conf)
        self.check('user-city: "010"', conf)



# -*- coding: utf-8 -*-
# yamllint documentation build configuration file, created by
# sphinx-quickstart on Thu Jan 21 21:18:52 2016.

import sys
import os
from unittest.mock import MagicMock

sys.path.insert(0, os.path.abspath('..'))  # noqa

from yamllint import __copyright__, APP_NAME, APP_VERSION

# -- General configuration ------------------------------------------------

extensions = [
    'sphinx.ext.autodoc',
]

source_suffix = '.rst'

master_doc = 'index'

project = APP_NAME
copyright = __copyright__

version = APP_VERSION
release = APP_VERSION

pygments_style = 'sphinx'

# -- Options for HTML output ----------------------------------------------

html_theme = 'default'

htmlhelp_basename = 'yamllintdoc'

# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'yamllint', '', [u'Adrien Vergé'], 1)
]

# -- Build with sphinx automodule without needing to install third-party libs


class Mock(MagicMock):
    @classmethod
    def __getattr__(cls, name):
        return MagicMock()


MOCK_MODULES = ['pathspec', 'yaml']
sys.modules.update((mod_name, Mock()) for mod_name in MOCK_MODULES)



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class CommentsTestCase(RuleTestCase):
    rule_id = 'comments'

    def test_disabled(self):
        conf = ('comments: disable\n'
                'comments-indentation: disable\n')
        self.check('---\n'
                   '#comment\n'
                   '\n'
                   'test: #    description\n'
                   '  - foo  # bar\n'
                   '  - hello #world\n'
                   '\n'
                   '# comment 2\n'
                   '#comment 3\n'
                   '  #comment 3 bis\n'
                   '  #  comment 3 ter\n'
                   '\n'
                   '################################\n'
                   '## comment 4\n'
                   '##comment 5\n'
                   '\n'
                   'string: "Une longue phrase." # this is French\n', conf)

    def test_starting_space(self):
        conf = ('comments:\n'
                '  require-starting-space: true\n'
                '  min-spaces-from-content: -1\n'
                'comments-indentation: disable\n')
        self.check('---\n'
                   '# comment\n'
                   '\n'
                   'test:  #     description\n'
                   '  - foo  #   bar\n'
                   '  - hello  # world\n'
                   '\n'
                   '# comment 2\n'
                   '# comment 3\n'
                   '  #  comment 3 bis\n'
                   '  #  comment 3 ter\n'
                   '\n'
                   '################################\n'
                   '## comment 4\n'
                   '##  comment 5\n', conf)
        self.check('---\n'
                   '#comment\n'
                   '\n'
                   'test:  #    description\n'
                   '  - foo  #  bar\n'
                   '  - hello  #world\n'
                   '\n'
                   '# comment 2\n'
                   '#comment 3\n'
                   '  #comment 3 bis\n'
                   '  #  comment 3 ter\n'
                   '\n'
                   '################################\n'
                   '## comment 4\n'
                   '##comment 5\n', conf,
                   problem1=(2, 2), problem2=(6, 13),
                   problem3=(9, 2), problem4=(10, 4),
                   problem5=(15, 3))

    def test_shebang(self):
        conf = ('comments:\n'
                '  require-starting-space: true\n'
                '  ignore-shebangs: false\n'
                'comments-indentation: disable\n'
                'document-start: disable\n')
        self.check('#!/bin/env my-interpreter\n',
                   conf, problem1=(1, 2))
        self.check('# comment\n'
                   '#!/bin/env my-interpreter\n', conf,
                   problem1=(2, 2))
        self.check('#!/bin/env my-interpreter\n'
                   '---\n'
                   '#comment\n'
                   '#!/bin/env my-interpreter\n'
                   '', conf,
                   problem1=(1, 2), problem2=(3, 2), problem3=(4, 2))
        self.check('#! not a shebang\n',
                   conf, problem1=(1, 2))
        self.check('key:  #!/not/a/shebang\n',
                   conf, problem1=(1, 8))

    def test_ignore_shebang(self):
        conf = ('comments:\n'
                '  require-starting-space: true\n'
                '  ignore-shebangs: true\n'
                'comments-indentation: disable\n'
                'document-start: disable\n')
        self.check('#!/bin/env my-interpreter\n', conf)
        self.check('# comment\n'
                   '#!/bin/env my-interpreter\n', conf,
                   problem1=(2, 2))
        self.check('#!/bin/env my-interpreter\n'
                   '---\n'
                   '#comment\n'
                   '#!/bin/env my-interpreter\n', conf,
                   problem2=(3, 2), problem3=(4, 2))
        self.check('#! not a shebang\n',
                   conf, problem1=(1, 2))
        self.check('key:  #!/not/a/shebang\n',
                   conf, problem1=(1, 8))

    def test_spaces_from_content(self):
        conf = ('comments:\n'
                '  require-starting-space: false\n'
                '  min-spaces-from-content: 2\n')
        self.check('---\n'
                   '# comment\n'
                   '\n'
                   'test:  #    description\n'
                   '  - foo  #  bar\n'
                   '  - hello  #world\n'
                   '\n'
                   'string: "Une longue phrase."  # this is French\n', conf)
        self.check('---\n'
                   '# comment\n'
                   '\n'
                   'test: #    description\n'
                   '  - foo  # bar\n'
                   '  - hello #world\n'
                   '\n'
                   'string: "Une longue phrase." # this is French\n', conf,
                   problem1=(4, 7), problem2=(6, 11), problem3=(8, 30))

    def test_both(self):
        conf = ('comments:\n'
                '  require-starting-space: true\n'
                '  min-spaces-from-content: 2\n'
                'comments-indentation: disable\n')
        self.check('---\n'
                   '#comment\n'
                   '\n'
                   'test: #    description\n'
                   '  - foo  # bar\n'
                   '  - hello #world\n'
                   '\n'
                   '# comment 2\n'
                   '#comment 3\n'
                   '  #comment 3 bis\n'
                   '  #  comment 3 ter\n'
                   '\n'
                   '################################\n'
                   '## comment 4\n'
                   '##comment 5\n'
                   '\n'
                   'string: "Une longue phrase." # this is French\n', conf,
                   problem1=(2, 2),
                   problem2=(4, 7),
                   problem3=(6, 11), problem4=(6, 12),
                   problem5=(9, 2),
                   problem6=(10, 4),
                   problem7=(15, 3),
                   problem8=(17, 30))

    def test_empty_comment(self):
        conf = ('comments:\n'
                '  require-starting-space: true\n'
                '  min-spaces-from-content: 2\n')
        self.check('---\n'
                   '# This is paragraph 1.\n'
                   '#\n'
                   '# This is paragraph 2.\n', conf)
        self.check('---\n'
                   'inline: comment  #\n'
                   'foo: bar\n', conf)

    def test_first_line(self):
        conf = ('comments:\n'
                '  require-starting-space: true\n'
                '  min-spaces-from-content: 2\n')
        self.check('# comment\n', conf)

    def test_last_line(self):
        conf = ('comments:\n'
                '  require-starting-space: true\n'
                '  min-spaces-from-content: 2\n'
                'new-line-at-end-of-file: disable\n')
        self.check('# comment with no newline char:\n'
                   '#', conf)

    def test_multi_line_scalar(self):
        conf = ('comments:\n'
                '  require-starting-space: true\n'
                '  min-spaces-from-content: 2\n'
                'trailing-spaces: disable\n')
        self.check('---\n'
                   'string: >\n'
                   '  this is plain text\n'
                   '\n'
                   '# comment\n', conf)
        self.check('---\n'
                   '- string: >\n'
                   '    this is plain text\n'
                   '  \n'
                   '  # comment\n', conf)



# -*- coding: utf-8 -*-
# Copyright (C) 2017 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
import subprocess
import tempfile
import sys
import unittest


PYTHON = sys.executable or 'python'


class ModuleTestCase(unittest.TestCase):
    def setUp(self):
        self.wd = tempfile.mkdtemp(prefix='yamllint-tests-')

        # file with only one warning
        with open(os.path.join(self.wd, 'warn.yaml'), 'w') as f:
            f.write('key: value\n')

        # file in dir
        os.mkdir(os.path.join(self.wd, 'sub'))
        with open(os.path.join(self.wd, 'sub', 'nok.yaml'), 'w') as f:
            f.write('---\n'
                    'list: [  1, 1, 2, 3, 5, 8]  \n')

    def tearDown(self):
        shutil.rmtree(self.wd)

    def test_run_module_no_args(self):
        with self.assertRaises(subprocess.CalledProcessError) as ctx:
            subprocess.check_output([PYTHON, '-m', 'yamllint'],
                                    stderr=subprocess.STDOUT)
        self.assertEqual(ctx.exception.returncode, 2)
        self.assertRegexpMatches(ctx.exception.output.decode(),
                                 r'^usage: yamllint')

    def test_run_module_on_bad_dir(self):
        with self.assertRaises(subprocess.CalledProcessError) as ctx:
            subprocess.check_output([PYTHON, '-m', 'yamllint',
                                     '/does/not/exist'],
                                    stderr=subprocess.STDOUT)
        self.assertRegexpMatches(ctx.exception.output.decode(),
                                 r'No such file or directory')

    def test_run_module_on_file(self):
        out = subprocess.check_output(
            [PYTHON, '-m', 'yamllint', os.path.join(self.wd, 'warn.yaml')])
        lines = out.decode().splitlines()
        self.assertIn('/warn.yaml', lines[0])
        self.assertEqual('\n'.join(lines[1:]),
                         '  1:1       warning  missing document start "---"'
                         '  (document-start)\n')

    def test_run_module_on_dir(self):
        with self.assertRaises(subprocess.CalledProcessError) as ctx:
            subprocess.check_output([PYTHON, '-m', 'yamllint', self.wd])
        self.assertEqual(ctx.exception.returncode, 1)

        files = ctx.exception.output.decode().split('\n\n')
        self.assertIn(
            '/warn.yaml\n'
            '  1:1       warning  missing document start "---"'
            '  (document-start)',
            files[0])
        self.assertIn(
            '/sub/nok.yaml\n'
            '  2:9       error    too many spaces inside brackets'
            '  (brackets)\n'
            '  2:27      error    trailing spaces  (trailing-spaces)',
            files[1])



# -*- coding: utf-8 -*-
# Copyright (C) 2018 ClearScore
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to forbid any string values that are not quoted.
You can also enforce the type of the quote used using the ``quote-type`` option
(``single``, ``double`` or ``any``).

**Note**: Multi-line strings (with ``|`` or ``>``) will not be checked.

.. rubric:: Examples

#. With ``quoted-strings: {quote-type: any}``

   the following code snippet would **PASS**:
   ::

    foo: "bar"
    bar: 'foo'
    number: 123
    boolean: true

   the following code snippet would **FAIL**:
   ::

    foo: bar
"""

import yaml

from yamllint.linter import LintProblem

ID = 'quoted-strings'
TYPE = 'token'
CONF = {'quote-type': ('any', 'single', 'double')}
DEFAULT = {'quote-type': 'any'}


def check(conf, token, prev, next, nextnext, context):
    quote_type = conf['quote-type']

    if (isinstance(token, yaml.tokens.ScalarToken) and
            isinstance(prev, (yaml.ValueToken, yaml.TagToken))):
        # Ignore explicit types, e.g. !!str testtest or !!int 42
        if (prev and isinstance(prev, yaml.tokens.TagToken) and
                prev.value[0] == '!!'):
            return

        # Ignore numbers, booleans, etc.
        resolver = yaml.resolver.Resolver()
        if resolver.resolve(yaml.nodes.ScalarNode, token.value,
                            (True, False)) != 'tag:yaml.org,2002:str':
            return

        # Ignore multi-line strings
        if (not token.plain) and (token.style == "|" or token.style == ">"):
            return

        if ((quote_type == 'single' and token.style != "'") or
                (quote_type == 'double' and token.style != '"') or
                (quote_type == 'any' and token.style is None)):
            yield LintProblem(
                token.start_mark.line + 1,
                token.start_mark.column + 1,
                "string value is not quoted with %s quotes" % (quote_type))



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class EmptyLinesTestCase(RuleTestCase):
    rule_id = 'empty-lines'

    def test_disabled(self):
        conf = ('empty-lines: disable\n'
                'new-line-at-end-of-file: disable\n'
                'document-start: disable\n')
        self.check('', conf)
        self.check('\n', conf)
        self.check('\n\n', conf)
        self.check('\n\n\n\n\n\n\n\n\n', conf)
        self.check('some text\n\n\n\n\n\n\n\n\n', conf)
        self.check('\n\n\n\n\n\n\n\n\nsome text', conf)
        self.check('\n\n\nsome text\n\n\n', conf)

    def test_empty_document(self):
        conf = ('empty-lines: {max: 0, max-start: 0, max-end: 0}\n'
                'new-line-at-end-of-file: disable\n'
                'document-start: disable\n')
        self.check('', conf)
        self.check('\n', conf)

    def test_0_empty_lines(self):
        conf = ('empty-lines: {max: 0, max-start: 0, max-end: 0}\n'
                'new-line-at-end-of-file: disable\n')
        self.check('---\n', conf)
        self.check('---\ntext\n\ntext', conf, problem=(3, 1))
        self.check('---\ntext\n\ntext\n', conf, problem=(3, 1))

    def test_10_empty_lines(self):
        conf = 'empty-lines: {max: 10, max-start: 0, max-end: 0}'
        self.check('---\nintro\n\n\n\n\n\n\n\n\n\n\nconclusion\n', conf)
        self.check('---\nintro\n\n\n\n\n\n\n\n\n\n\n\nconclusion\n', conf,
                   problem=(13, 1))

    def test_spaces(self):
        conf = ('empty-lines: {max: 1, max-start: 0, max-end: 0}\n'
                'trailing-spaces: disable\n')
        self.check('---\nintro\n\n \n\nconclusion\n', conf)
        self.check('---\nintro\n\n \n\n\nconclusion\n', conf, problem=(6, 1))

    def test_empty_lines_at_start(self):
        conf = ('empty-lines: {max: 2, max-start: 4, max-end: 0}\n'
                'document-start: disable\n')
        self.check('\n\n\n\nnon empty\n', conf)
        self.check('\n\n\n\n\nnon empty\n', conf, problem=(5, 1))

        conf = ('empty-lines: {max: 2, max-start: 0, max-end: 0}\n'
                'document-start: disable\n')
        self.check('non empty\n', conf)
        self.check('\nnon empty\n', conf, problem=(1, 1))

    def test_empty_lines_at_end(self):
        conf = ('empty-lines: {max: 2, max-start: 0, max-end: 4}\n'
                'document-start: disable\n')
        self.check('non empty\n\n\n\n\n', conf)
        self.check('non empty\n\n\n\n\n\n', conf, problem=(6, 1))
        conf = ('empty-lines: {max: 2, max-start: 0, max-end: 0}\n'
                'document-start: disable\n')
        self.check('non empty\n', conf)
        self.check('non empty\n\n', conf, problem=(2, 1))

    def test_with_dos_newlines(self):
        conf = ('empty-lines: {max: 2, max-start: 0, max-end: 0}\n'
                'new-lines: {type: dos}\n'
                'document-start: disable\n')
        self.check('---\r\n', conf)
        self.check('---\r\ntext\r\n\r\ntext\r\n', conf)
        self.check('\r\n---\r\ntext\r\n\r\ntext\r\n', conf,
                   problem=(1, 1))
        self.check('\r\n\r\n\r\n---\r\ntext\r\n\r\ntext\r\n', conf,
                   problem=(3, 1))
        self.check('---\r\ntext\r\n\r\n\r\n\r\ntext\r\n', conf,
                   problem=(5, 1))
        self.check('---\r\ntext\r\n\r\n\r\n\r\n\r\n\r\n\r\ntext\r\n', conf,
                   problem=(8, 1))
        self.check('---\r\ntext\r\n\r\ntext\r\n\r\n', conf,
                   problem=(5, 1))
        self.check('---\r\ntext\r\n\r\ntext\r\n\r\n\r\n\r\n', conf,
                   problem=(7, 1))



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to require or forbid the use of document start marker (``---``).

.. rubric:: Options

* Set ``present`` to ``true`` when the document start marker is required, or to
  ``false`` when it is forbidden.

.. rubric:: Examples

#. With ``document-start: {present: true}``

   the following code snippet would **PASS**:
   ::

    ---
    this:
      is: [a, document]
    ---
    - this
    - is: another one

   the following code snippet would **FAIL**:
   ::

    this:
      is: [a, document]
    ---
    - this
    - is: another one

#. With ``document-start: {present: false}``

   the following code snippet would **PASS**:
   ::

    this:
      is: [a, document]
    ...

   the following code snippet would **FAIL**:
   ::

    ---
    this:
      is: [a, document]
    ...
"""


import yaml

from yamllint.linter import LintProblem


ID = 'document-start'
TYPE = 'token'
CONF = {'present': bool}
DEFAULT = {'present': True}


def check(conf, token, prev, next, nextnext, context):
    if conf['present']:
        if (isinstance(prev, (yaml.StreamStartToken,
                              yaml.DocumentEndToken,
                              yaml.DirectiveToken)) and
            not isinstance(token, (yaml.DocumentStartToken,
                                   yaml.DirectiveToken,
                                   yaml.StreamEndToken))):
            yield LintProblem(token.start_mark.line + 1, 1,
                              'missing document start "---"')

    else:
        if isinstance(token, yaml.DocumentStartToken):
            yield LintProblem(token.start_mark.line + 1,
                              token.start_mark.column + 1,
                              'found forbidden document start "---"')



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import tempfile
import unittest

import yaml

from yamllint.config import YamlLintConfig
from yamllint import linter


class RuleTestCase(unittest.TestCase):
    def build_fake_config(self, conf):
        if conf is None:
            conf = {}
        else:
            conf = yaml.safe_load(conf)
        conf = {'extends': 'default',
                'rules': conf}
        return YamlLintConfig(yaml.safe_dump(conf))

    def check(self, source, conf, **kwargs):
        expected_problems = []
        for key in kwargs:
            assert key.startswith('problem')
            if len(kwargs[key]) > 2:
                if kwargs[key][2] == 'syntax':
                    rule_id = None
                else:
                    rule_id = kwargs[key][2]
            else:
                rule_id = self.rule_id
            expected_problems.append(linter.LintProblem(
                kwargs[key][0], kwargs[key][1], rule=rule_id))
        expected_problems.sort()

        real_problems = list(linter.run(source, self.build_fake_config(conf)))
        self.assertEqual(real_problems, expected_problems)


def build_temp_workspace(files):
    tempdir = tempfile.mkdtemp(prefix='yamllint-tests-')

    for path, content in files.items():
        path = os.path.join(tempdir, path)
        if not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))

        if type(content) is list:
            os.mkdir(path)
        else:
            mode = 'wb' if isinstance(content, bytes) else 'w'
            with open(path, mode) as f:
                f.write(content)

    return tempdir



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Peter Ericson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class TruthyTestCase(RuleTestCase):
    rule_id = 'truthy'

    def test_disabled(self):
        conf = 'truthy: disable'
        self.check('---\n'
                   '1: True\n', conf)
        self.check('---\n'
                   'True: 1\n', conf)

    def test_enabled(self):
        conf = 'truthy: enable\n'
        self.check('---\n'
                   '1: True\n'
                   'True: 1\n',
                   conf, problem1=(2, 4), problem2=(3, 1))
        self.check('---\n'
                   '1: "True"\n'
                   '"True": 1\n', conf)
        self.check('---\n'
                   '[\n'
                   '  true, false,\n'
                   '  "false", "FALSE",\n'
                   '  "true", "True",\n'
                   '  True, FALSE,\n'
                   '  on, OFF,\n'
                   '  NO, Yes\n'
                   ']\n', conf,
                   problem1=(6, 3), problem2=(6, 9),
                   problem3=(7, 3), problem4=(7, 7),
                   problem5=(8, 3), problem6=(8, 7))

    def test_different_allowed_values(self):
        conf = ('truthy:\n'
                '  allowed-values: ["yes", "no"]\n')
        self.check('---\n'
                   'key1: foo\n'
                   'key2: yes\n'
                   'key3: bar\n'
                   'key4: no\n', conf)
        self.check('---\n'
                   'key1: true\n'
                   'key2: Yes\n'
                   'key3: false\n'
                   'key4: no\n'
                   'key5: yes\n',
                   conf,
                   problem1=(2, 7), problem2=(3, 7),
                   problem3=(4, 7))

    def test_combined_allowed_values(self):
        conf = ('truthy:\n'
                '  allowed-values: ["yes", "no", "true", "false"]\n')
        self.check('---\n'
                   'key1: foo\n'
                   'key2: yes\n'
                   'key3: bar\n'
                   'key4: no\n', conf)
        self.check('---\n'
                   'key1: true\n'
                   'key2: Yes\n'
                   'key3: false\n'
                   'key4: no\n'
                   'key5: yes\n',
                   conf, problem1=(3, 7))

    def test_no_allowed_values(self):
        conf = ('truthy:\n'
                '  allowed-values: []\n')
        self.check('---\n'
                   'key1: foo\n'
                   'key2: bar\n', conf)
        self.check('---\n'
                   'key1: true\n'
                   'key2: yes\n'
                   'key3: false\n'
                   'key4: no\n', conf,
                   problem1=(2, 7), problem2=(3, 7),
                   problem3=(4, 7), problem4=(5, 7))

    def test_explicit_types(self):
        conf = 'truthy: enable\n'
        self.check('---\n'
                   'string1: !!str True\n'
                   'string2: !!str yes\n'
                   'string3: !!str off\n'
                   'encoded: !!binary |\n'
                   '           True\n'
                   '           OFF\n'
                   '           pad==\n'  # this decodes as 'N\xbb\x9e8Qii'
                   'boolean1: !!bool true\n'
                   'boolean2: !!bool "false"\n'
                   'boolean3: !!bool FALSE\n'
                   'boolean4: !!bool True\n'
                   'boolean5: !!bool off\n'
                   'boolean6: !!bool NO\n',
                   conf)



# -*- coding: utf-8 -*-
# Copyright (C) 2018 ClearScore
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class QuotedTestCase(RuleTestCase):
    rule_id = 'quoted-strings'

    def test_disabled(self):
        conf = 'quoted-strings: disable'
        self.check('---\n'
                   'foo: bar\n', conf)
        self.check('---\n'
                   'foo: "bar"\n', conf)
        self.check('---\n'
                   'foo: \'bar\'\n', conf)
        self.check('---\n'
                   'bar: 123\n', conf)

    def test_quote_type_any(self):
        conf = 'quoted-strings: {quote-type: any}\n'
        self.check('---\n'
                   'boolean1: true\n'
                   'number1: 123\n'
                   'string1: foo\n'                          # fails
                   'string2: "foo"\n'
                   'string3: \'bar\'\n'
                   'string4: !!str genericstring\n'
                   'string5: !!str 456\n'
                   'string6: !!str "quotedgenericstring"\n'
                   'binary: !!binary binstring\n'
                   'integer: !!int intstring\n'
                   'boolean2: !!bool boolstring\n'
                   'boolean3: !!bool "quotedboolstring"\n',
                   conf, problem=(4, 10))
        self.check('---\n'
                   'multiline string 1: |\n'
                   '  line 1\n'
                   '  line 2\n'
                   'multiline string 2: >\n'
                   '  word 1\n'
                   '  word 2\n'
                   'multiline string 3:\n'
                   '  word 1\n'
                   '  word 2\n'
                   'multiline string 4:\n'
                   '  "word 1\\\n'
                   '   word 2"\n',
                   conf, problem1=(9, 3))

    def test_quote_type_single(self):
        conf = 'quoted-strings: {quote-type: single}\n'
        self.check('---\n'
                   'boolean1: true\n'
                   'number1: 123\n'
                   'string1: foo\n'                          # fails
                   'string2: "foo"\n'                        # fails
                   'string3: \'bar\'\n'
                   'string4: !!str genericstring\n'
                   'string5: !!str 456\n'
                   'string6: !!str "quotedgenericstring"\n'
                   'binary: !!binary binstring\n'
                   'integer: !!int intstring\n'
                   'boolean2: !!bool boolstring\n'
                   'boolean3: !!bool "quotedboolstring"\n',
                   conf, problem1=(4, 10), problem2=(5, 10))
        self.check('---\n'
                   'multiline string 1: |\n'
                   '  line 1\n'
                   '  line 2\n'
                   'multiline string 2: >\n'
                   '  word 1\n'
                   '  word 2\n'
                   'multiline string 3:\n'
                   '  word 1\n'
                   '  word 2\n'
                   'multiline string 4:\n'
                   '  "word 1\\\n'
                   '   word 2"\n',
                   conf, problem1=(9, 3), problem2=(12, 3))

    def test_quote_type_double(self):
        conf = 'quoted-strings: {quote-type: double}\n'
        self.check('---\n'
                   'boolean1: true\n'
                   'number1: 123\n'
                   'string1: foo\n'                          # fails
                   'string2: "foo"\n'
                   'string3: \'bar\'\n'                      # fails
                   'string4: !!str genericstring\n'
                   'string5: !!str 456\n'
                   'string6: !!str "quotedgenericstring"\n'
                   'binary: !!binary binstring\n'
                   'integer: !!int intstring\n'
                   'boolean2: !!bool boolstring\n'
                   'boolean3: !!bool "quotedboolstring"\n',
                   conf, problem1=(4, 10), problem2=(6, 10))
        self.check('---\n'
                   'multiline string 1: |\n'
                   '  line 1\n'
                   '  line 2\n'
                   'multiline string 2: >\n'
                   '  word 1\n'
                   '  word 2\n'
                   'multiline string 3:\n'
                   '  word 1\n'
                   '  word 2\n'
                   'multiline string 4:\n'
                   '  "word 1\\\n'
                   '   word 2"\n',
                   conf, problem1=(9, 3))



# -*- coding: utf-8 -*-
# Copyright (C) 2017 Greg Dubicki
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class EmptyValuesTestCase(RuleTestCase):
    rule_id = 'empty-values'

    def test_disabled(self):
        conf = ('empty-values: disable\n'
                'braces: disable\n'
                'commas: disable\n')
        self.check('---\n'
                   'foo:\n', conf)
        self.check('---\n'
                   'foo:\n'
                   ' bar:\n', conf)
        self.check('---\n'
                   '{a:}\n', conf)
        self.check('---\n'
                   'foo: {a:}\n', conf)
        self.check('---\n'
                   '- {a:}\n'
                   '- {a:, b: 2}\n'
                   '- {a: 1, b:}\n'
                   '- {a: 1, b: , }\n', conf)
        self.check('---\n'
                   '{a: {b: , c: {d: 4, e:}}, f:}\n', conf)

    def test_in_block_mappings_disabled(self):
        conf = ('empty-values: {forbid-in-block-mappings: false,\n'
                '               forbid-in-flow-mappings: false}\n')
        self.check('---\n'
                   'foo:\n', conf)
        self.check('---\n'
                   'foo:\n'
                   'bar: aaa\n', conf)

    def test_in_block_mappings_single_line(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n')
        self.check('---\n'
                   'implicitly-null:\n', conf, problem1=(2, 17))
        self.check('---\n'
                   'implicitly-null:with-colons:in-key:\n', conf,
                   problem1=(2, 36))
        self.check('---\n'
                   'implicitly-null:with-colons:in-key2:\n', conf,
                   problem1=(2, 37))

    def test_in_block_mappings_all_lines(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n')
        self.check('---\n'
                   'foo:\n'
                   'bar:\n'
                   'foobar:\n', conf, problem1=(2, 5),
                   problem2=(3, 5), problem3=(4, 8))

    def test_in_block_mappings_explicit_end_of_document(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n')
        self.check('---\n'
                   'foo:\n'
                   '...\n', conf, problem1=(2, 5))

    def test_in_block_mappings_not_end_of_document(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n')
        self.check('---\n'
                   'foo:\n'
                   'bar:\n'
                   ' aaa\n', conf, problem1=(2, 5))

    def test_in_block_mappings_different_level(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n')
        self.check('---\n'
                   'foo:\n'
                   ' bar:\n'
                   'aaa: bbb\n', conf, problem1=(3, 6))

    def test_in_block_mappings_empty_flow_mapping(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n'
                'braces: disable\n'
                'commas: disable\n')
        self.check('---\n'
                   'foo: {a:}\n', conf)
        self.check('---\n'
                   '- {a:, b: 2}\n'
                   '- {a: 1, b:}\n'
                   '- {a: 1, b: , }\n', conf)

    def test_in_block_mappings_empty_block_sequence(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n')
        self.check('---\n'
                   'foo:\n'
                   '  -\n', conf)

    def test_in_block_mappings_not_empty_or_explicit_null(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n')
        self.check('---\n'
                   'foo:\n'
                   ' bar:\n'
                   '  aaa\n', conf)
        self.check('---\n'
                   'explicitly-null: null\n', conf)
        self.check('---\n'
                   'explicitly-null:with-colons:in-key: null\n', conf)
        self.check('---\n'
                   'false-null: nulL\n', conf)
        self.check('---\n'
                   'empty-string: \'\'\n', conf)
        self.check('---\n'
                   'nullable-boolean: false\n', conf)
        self.check('---\n'
                   'nullable-int: 0\n', conf)
        self.check('---\n'
                   'First occurrence: &anchor Foo\n'
                   'Second occurrence: *anchor\n', conf)

    def test_in_block_mappings_various_explicit_null(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n')
        self.check('---\n'
                   'null-alias: ~\n', conf)
        self.check('---\n'
                   'null-key1: {?: val}\n', conf)
        self.check('---\n'
                   'null-key2: {? !!null "": val}\n', conf)

    def test_in_block_mappings_comments(self):
        conf = ('empty-values: {forbid-in-block-mappings: true,\n'
                '               forbid-in-flow-mappings: false}\n'
                'comments: disable\n')
        self.check('---\n'
                   'empty:  # comment\n'
                   'foo:\n'
                   '  bar: # comment\n', conf,
                   problem1=(2, 7),
                   problem2=(4, 7))

    def test_in_flow_mappings_disabled(self):
        conf = ('empty-values: {forbid-in-block-mappings: false,\n'
                '               forbid-in-flow-mappings: false}\n'
                'braces: disable\n'
                'commas: disable\n')
        self.check('---\n'
                   '{a:}\n', conf)
        self.check('---\n'
                   'foo: {a:}\n', conf)
        self.check('---\n'
                   '- {a:}\n'
                   '- {a:, b: 2}\n'
                   '- {a: 1, b:}\n'
                   '- {a: 1, b: , }\n', conf)
        self.check('---\n'
                   '{a: {b: , c: {d: 4, e:}}, f:}\n', conf)

    def test_in_flow_mappings_single_line(self):
        conf = ('empty-values: {forbid-in-block-mappings: false,\n'
                '               forbid-in-flow-mappings: true}\n'
                'braces: disable\n'
                'commas: disable\n')
        self.check('---\n'
                   '{a:}\n', conf,
                   problem=(2, 4))
        self.check('---\n'
                   'foo: {a:}\n', conf,
                   problem=(2, 9))
        self.check('---\n'
                   '- {a:}\n'
                   '- {a:, b: 2}\n'
                   '- {a: 1, b:}\n'
                   '- {a: 1, b: , }\n', conf,
                   problem1=(2, 6),
                   problem2=(3, 6),
                   problem3=(4, 12),
                   problem4=(5, 12))
        self.check('---\n'
                   '{a: {b: , c: {d: 4, e:}}, f:}\n', conf,
                   problem1=(2, 8),
                   problem2=(2, 23),
                   problem3=(2, 29))

    def test_in_flow_mappings_multi_line(self):
        conf = ('empty-values: {forbid-in-block-mappings: false,\n'
                '               forbid-in-flow-mappings: true}\n'
                'braces: disable\n'
                'commas: disable\n')
        self.check('---\n'
                   'foo: {\n'
                   '  a:\n'
                   '}\n', conf,
                   problem=(3, 5))
        self.check('---\n'
                   '{\n'
                   '  a: {\n'
                   '    b: ,\n'
                   '    c: {\n'
                   '      d: 4,\n'
                   '      e:\n'
                   '    }\n'
                   '  },\n'
                   '  f:\n'
                   '}\n', conf,
                   problem1=(4, 7),
                   problem2=(7, 9),
                   problem3=(10, 5))

    def test_in_flow_mappings_various_explicit_null(self):
        conf = ('empty-values: {forbid-in-block-mappings: false,\n'
                '               forbid-in-flow-mappings: true}\n'
                'braces: disable\n'
                'commas: disable\n')
        self.check('---\n'
                   '{explicit-null: null}\n', conf)
        self.check('---\n'
                   '{null-alias: ~}\n', conf)
        self.check('---\n'
                   'null-key1: {?: val}\n', conf)
        self.check('---\n'
                   'null-key2: {? !!null "": val}\n', conf)

    def test_in_flow_mappings_comments(self):
        conf = ('empty-values: {forbid-in-block-mappings: false,\n'
                '               forbid-in-flow-mappings: true}\n'
                'braces: disable\n'
                'commas: disable\n'
                'comments: disable\n')
        self.check('---\n'
                   '{\n'
                   '  a: {\n'
                   '    b: ,  # comment\n'
                   '    c: {\n'
                   '      d: 4,  # comment\n'
                   '      e:  # comment\n'
                   '    }\n'
                   '  },\n'
                   '  f:  # comment\n'
                   '}\n', conf,
                   problem1=(4, 7),
                   problem2=(7, 9),
                   problem3=(10, 5))



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to control the number of spaces inside braces (``{`` and ``}``).

.. rubric:: Options

* ``min-spaces-inside`` defines the minimal number of spaces required inside
  braces.
* ``max-spaces-inside`` defines the maximal number of spaces allowed inside
  braces.
* ``min-spaces-inside-empty`` defines the minimal number of spaces required
  inside empty braces.
* ``max-spaces-inside-empty`` defines the maximal number of spaces allowed
  inside empty braces.

.. rubric:: Examples

#. With ``braces: {min-spaces-inside: 0, max-spaces-inside: 0}``

   the following code snippet would **PASS**:
   ::

    object: {key1: 4, key2: 8}

   the following code snippet would **FAIL**:
   ::

    object: { key1: 4, key2: 8 }

#. With ``braces: {min-spaces-inside: 1, max-spaces-inside: 3}``

   the following code snippet would **PASS**:
   ::

    object: { key1: 4, key2: 8 }

   the following code snippet would **PASS**:
   ::

    object: { key1: 4, key2: 8   }

   the following code snippet would **FAIL**:
   ::

    object: {    key1: 4, key2: 8   }

   the following code snippet would **FAIL**:
   ::

    object: {key1: 4, key2: 8 }

#. With ``braces: {min-spaces-inside-empty: 0, max-spaces-inside-empty: 0}``

   the following code snippet would **PASS**:
   ::

    object: {}

   the following code snippet would **FAIL**:
   ::

    object: { }

#. With ``braces: {min-spaces-inside-empty: 1, max-spaces-inside-empty: -1}``

   the following code snippet would **PASS**:
   ::

    object: {         }

   the following code snippet would **FAIL**:
   ::

    object: {}
"""


import yaml

from yamllint.rules.common import spaces_after, spaces_before


ID = 'braces'
TYPE = 'token'
CONF = {'min-spaces-inside': int,
        'max-spaces-inside': int,
        'min-spaces-inside-empty': int,
        'max-spaces-inside-empty': int}
DEFAULT = {'min-spaces-inside': 0,
           'max-spaces-inside': 0,
           'min-spaces-inside-empty': -1,
           'max-spaces-inside-empty': -1}


def check(conf, token, prev, next, nextnext, context):
    if (isinstance(token, yaml.FlowMappingStartToken) and
            isinstance(next, yaml.FlowMappingEndToken)):
        problem = spaces_after(token, prev, next,
                               min=(conf['min-spaces-inside-empty']
                                    if conf['min-spaces-inside-empty'] != -1
                                    else conf['min-spaces-inside']),
                               max=(conf['max-spaces-inside-empty']
                                    if conf['max-spaces-inside-empty'] != -1
                                    else conf['max-spaces-inside']),
                               min_desc='too few spaces inside empty braces',
                               max_desc='too many spaces inside empty braces')
        if problem is not None:
            yield problem

    elif isinstance(token, yaml.FlowMappingStartToken):
        problem = spaces_after(token, prev, next,
                               min=conf['min-spaces-inside'],
                               max=conf['max-spaces-inside'],
                               min_desc='too few spaces inside braces',
                               max_desc='too many spaces inside braces')
        if problem is not None:
            yield problem

    elif (isinstance(token, yaml.FlowMappingEndToken) and
            (prev is None or
             not isinstance(prev, yaml.FlowMappingStartToken))):
        problem = spaces_before(token, prev, next,
                                min=conf['min-spaces-inside'],
                                max=conf['max-spaces-inside'],
                                min_desc='too few spaces inside braces',
                                max_desc='too many spaces inside braces')
        if problem is not None:
            yield problem



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO
import fcntl
import locale
import os
import pty
import shutil
import sys
import unittest

from tests.common import build_temp_workspace

from yamllint import cli


class CommandLineTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        super(CommandLineTestCase, cls).setUpClass()

        cls.wd = build_temp_workspace({
            # .yaml file at root
            'a.yaml': '---\n'
                      '- 1   \n'
                      '- 2',
            # file with only one warning
            'warn.yaml': 'key: value\n',
            # .yml file at root
            'empty.yml': '',
            # file in dir
            'sub/ok.yaml': '---\n'
                           'key: value\n',
            # file in very nested dir
            's/s/s/s/s/s/s/s/s/s/s/s/s/s/s/file.yaml': '---\n'
                                                       'key: value\n'
                                                       'key: other value\n',
            # empty dir
            'empty-dir': [],
            # non-YAML file
            'no-yaml.json': '---\n'
                            'key: value\n',
            # non-ASCII chars
            'non-ascii/utf-8': (
                u'---\n'
                u'- hétérogénéité\n'
                u'# 19.99 €\n'
                u'- お早う御座います。\n'
                u'# الأَبْجَدِيَّة العَرَبِيَّة\n').encode('utf-8'),
        })

    @classmethod
    def tearDownClass(cls):
        super(CommandLineTestCase, cls).tearDownClass()

        shutil.rmtree(cls.wd)

    def test_find_files_recursively(self):
        self.assertEqual(
            sorted(cli.find_files_recursively([self.wd])),
            [os.path.join(self.wd, 'a.yaml'),
             os.path.join(self.wd, 'empty.yml'),
             os.path.join(self.wd, 's/s/s/s/s/s/s/s/s/s/s/s/s/s/s/file.yaml'),
             os.path.join(self.wd, 'sub/ok.yaml'),
             os.path.join(self.wd, 'warn.yaml')],
        )

        items = [os.path.join(self.wd, 'sub/ok.yaml'),
                 os.path.join(self.wd, 'empty-dir')]
        self.assertEqual(
            sorted(cli.find_files_recursively(items)),
            [os.path.join(self.wd, 'sub/ok.yaml')],
        )

        items = [os.path.join(self.wd, 'empty.yml'),
                 os.path.join(self.wd, 's')]
        self.assertEqual(
            sorted(cli.find_files_recursively(items)),
            [os.path.join(self.wd, 'empty.yml'),
             os.path.join(self.wd, 's/s/s/s/s/s/s/s/s/s/s/s/s/s/s/file.yaml')],
        )

        items = [os.path.join(self.wd, 'sub'),
                 os.path.join(self.wd, '/etc/another/file')]
        self.assertEqual(
            sorted(cli.find_files_recursively(items)),
            [os.path.join(self.wd, '/etc/another/file'),
             os.path.join(self.wd, 'sub/ok.yaml')],
        )

    def test_run_with_bad_arguments(self):
        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(())

        self.assertNotEqual(ctx.exception.code, 0)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertRegexpMatches(err, r'^usage')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('--unknown-arg', ))

        self.assertNotEqual(ctx.exception.code, 0)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertRegexpMatches(err, r'^usage')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-c', './conf.yaml', '-d', 'relaxed', 'file'))

        self.assertNotEqual(ctx.exception.code, 0)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertRegexpMatches(
            err.splitlines()[-1],
            r'^yamllint: error: argument -d\/--config-data: '
            r'not allowed with argument -c\/--config-file$'
        )

        # checks if reading from stdin and files are mutually exclusive
        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-', 'file'))

        self.assertNotEqual(ctx.exception.code, 0)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertRegexpMatches(err, r'^usage')

    def test_run_with_bad_config(self):
        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-d', 'rules: {a: b}', 'file'))

        self.assertEqual(ctx.exception.code, -1)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertRegexpMatches(err, r'^invalid config: no such rule')

    def test_run_with_empty_config(self):
        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-d', '', 'file'))

        self.assertEqual(ctx.exception.code, -1)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertRegexpMatches(err, r'^invalid config: not a dict')

    def test_run_with_config_file(self):
        with open(os.path.join(self.wd, 'config'), 'w') as f:
            f.write('rules: {trailing-spaces: disable}')

        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-c', f.name, os.path.join(self.wd, 'a.yaml')))
        self.assertEqual(ctx.exception.code, 0)

        with open(os.path.join(self.wd, 'config'), 'w') as f:
            f.write('rules: {trailing-spaces: enable}')

        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-c', f.name, os.path.join(self.wd, 'a.yaml')))
        self.assertEqual(ctx.exception.code, 1)

    def test_run_with_user_global_config_file(self):
        home = os.path.join(self.wd, 'fake-home')
        os.mkdir(home)
        dir = os.path.join(home, '.config')
        os.mkdir(dir)
        dir = os.path.join(dir, 'yamllint')
        os.mkdir(dir)
        config = os.path.join(dir, 'config')

        temp = os.environ['HOME']
        os.environ['HOME'] = home

        with open(config, 'w') as f:
            f.write('rules: {trailing-spaces: disable}')

        with self.assertRaises(SystemExit) as ctx:
            cli.run((os.path.join(self.wd, 'a.yaml'), ))
        self.assertEqual(ctx.exception.code, 0)

        with open(config, 'w') as f:
            f.write('rules: {trailing-spaces: enable}')

        with self.assertRaises(SystemExit) as ctx:
            cli.run((os.path.join(self.wd, 'a.yaml'), ))
        self.assertEqual(ctx.exception.code, 1)

        os.environ['HOME'] = temp

    def test_run_version(self):
        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('--version', ))

        self.assertEqual(ctx.exception.code, 0)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertRegexpMatches(out + err, r'yamllint \d+\.\d+')

    def test_run_non_existing_file(self):
        file = os.path.join(self.wd, 'i-do-not-exist.yaml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-f', 'parsable', file))

        self.assertEqual(ctx.exception.code, -1)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertRegexpMatches(err, r'No such file or directory')

    def test_run_one_problem_file(self):
        file = os.path.join(self.wd, 'a.yaml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-f', 'parsable', file))

        self.assertEqual(ctx.exception.code, 1)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, (
            '%s:2:4: [error] trailing spaces (trailing-spaces)\n'
            '%s:3:4: [error] no new line character at the end of file '
            '(new-line-at-end-of-file)\n') % (file, file))
        self.assertEqual(err, '')

    def test_run_one_warning(self):
        file = os.path.join(self.wd, 'warn.yaml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-f', 'parsable', file))

        self.assertEqual(ctx.exception.code, 0)

    def test_run_warning_in_strict_mode(self):
        file = os.path.join(self.wd, 'warn.yaml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-f', 'parsable', '--strict', file))

        self.assertEqual(ctx.exception.code, 2)

    def test_run_one_ok_file(self):
        file = os.path.join(self.wd, 'sub', 'ok.yaml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-f', 'parsable', file))

        self.assertEqual(ctx.exception.code, 0)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertEqual(err, '')

    def test_run_empty_file(self):
        file = os.path.join(self.wd, 'empty.yml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-f', 'parsable', file))

        self.assertEqual(ctx.exception.code, 0)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertEqual(err, '')

    def test_run_non_ascii_file(self):
        file = os.path.join(self.wd, 'non-ascii', 'utf-8')

        # Make sure the default localization conditions on this "system"
        # support UTF-8 encoding.
        loc = locale.getlocale()
        try:
            locale.setlocale(locale.LC_ALL, 'C.UTF-8')
        except locale.Error:
            locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-f', 'parsable', file))

        locale.setlocale(locale.LC_ALL, loc)

        self.assertEqual(ctx.exception.code, 0)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, '')
        self.assertEqual(err, '')

    def test_run_multiple_files(self):
        items = [os.path.join(self.wd, 'empty.yml'),
                 os.path.join(self.wd, 's')]
        file = items[1] + '/s/s/s/s/s/s/s/s/s/s/s/s/s/s/file.yaml'

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run(['-f', 'parsable'] + items)

        self.assertEqual(ctx.exception.code, 1)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, (
            '%s:3:1: [error] duplication of key "key" in mapping '
            '(key-duplicates)\n') % file)
        self.assertEqual(err, '')

    def test_run_piped_output_nocolor(self):
        file = os.path.join(self.wd, 'a.yaml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run((file, ))

        self.assertEqual(ctx.exception.code, 1)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, (
            '%s\n'
            '  2:4       error    trailing spaces  (trailing-spaces)\n'
            '  3:4       error    no new line character at the end of file  '
            '(new-line-at-end-of-file)\n'
            '\n' % file))
        self.assertEqual(err, '')

    def test_run_default_format_output_in_tty(self):
        file = os.path.join(self.wd, 'a.yaml')

        # Create a pseudo-TTY and redirect stdout to it
        master, slave = pty.openpty()
        sys.stdout = sys.stderr = os.fdopen(slave, 'w')

        with self.assertRaises(SystemExit) as ctx:
            cli.run((file, ))
        sys.stdout.flush()

        self.assertEqual(ctx.exception.code, 1)

        # Read output from TTY
        output = os.fdopen(master, 'r')
        flag = fcntl.fcntl(master, fcntl.F_GETFD)
        fcntl.fcntl(master, fcntl.F_SETFL, flag | os.O_NONBLOCK)

        out = output.read().replace('\r\n', '\n')

        sys.stdout.close()
        sys.stderr.close()
        output.close()

        self.assertEqual(out, (
            '\033[4m%s\033[0m\n'
            '  \033[2m2:4\033[0m       \033[31merror\033[0m    '
            'trailing spaces  \033[2m(trailing-spaces)\033[0m\n'
            '  \033[2m3:4\033[0m       \033[31merror\033[0m    '
            'no new line character at the end of file  '
            '\033[2m(new-line-at-end-of-file)\033[0m\n'
            '\n' % file))

    def test_run_default_format_output_without_tty(self):
        file = os.path.join(self.wd, 'a.yaml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run((file, ))

        self.assertEqual(ctx.exception.code, 1)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, (
            '%s\n'
            '  2:4       error    trailing spaces  (trailing-spaces)\n'
            '  3:4       error    no new line character at the end of file  '
            '(new-line-at-end-of-file)\n'
            '\n' % file))
        self.assertEqual(err, '')

    def test_run_auto_output_without_tty_output(self):
        file = os.path.join(self.wd, 'a.yaml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run((file, '--format', 'auto'))

        self.assertEqual(ctx.exception.code, 1)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, (
            '%s\n'
            '  2:4       error    trailing spaces  (trailing-spaces)\n'
            '  3:4       error    no new line character at the end of file  '
            '(new-line-at-end-of-file)\n'
            '\n' % file))
        self.assertEqual(err, '')

    def test_run_format_colored(self):
        file = os.path.join(self.wd, 'a.yaml')

        sys.stdout, sys.stderr = StringIO(), StringIO()
        with self.assertRaises(SystemExit) as ctx:
            cli.run((file, '--format', 'colored'))

        self.assertEqual(ctx.exception.code, 1)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, (
            '\033[4m%s\033[0m\n'
            '  \033[2m2:4\033[0m       \033[31merror\033[0m    '
            'trailing spaces  \033[2m(trailing-spaces)\033[0m\n'
            '  \033[2m3:4\033[0m       \033[31merror\033[0m    '
            'no new line character at the end of file  '
            '\033[2m(new-line-at-end-of-file)\033[0m\n'
            '\n' % file))
        self.assertEqual(err, '')

    def test_run_read_from_stdin(self):
        # prepares stdin with an invalid yaml string so that we can check
        # for its specific error, and be assured that stdin was read
        sys.stdout, sys.stderr = StringIO(), StringIO()
        sys.stdin = StringIO(
            'I am a string\n'
            'therefore: I am an error\n')

        with self.assertRaises(SystemExit) as ctx:
            cli.run(('-', '-f', 'parsable'))

        self.assertNotEqual(ctx.exception.code, 0)

        out, err = sys.stdout.getvalue(), sys.stderr.getvalue()
        self.assertEqual(out, (
            'stdin:2:10: [error] syntax error: '
            'mapping values are not allowed here\n'))
        self.assertEqual(err, '')



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase

from yamllint.parser import token_or_comment_generator, Comment
from yamllint.rules.indentation import check


class IndentationStackTestCase(RuleTestCase):
    # This test suite checks that the "indentation stack" built by the
    # indentation rule is valid. It is important, since everything else in the
    # rule relies on this stack.

    maxDiff = None

    def format_stack(self, stack):
        """Transform the stack at a given moment into a printable string like:

        B_MAP:0 KEY:0 VAL:5
        """
        return ' '.join(map(str, stack[1:]))

    def full_stack(self, source):
        conf = {'spaces': 2, 'indent-sequences': True,
                'check-multi-line-strings': False}
        context = {}
        output = ''
        for elem in [t for t in token_or_comment_generator(source)
                     if not isinstance(t, Comment)]:
            list(check(conf, elem.curr, elem.prev, elem.next, elem.nextnext,
                       context))

            token_type = (elem.curr.__class__.__name__
                          .replace('Token', '')
                          .replace('Block', 'B').replace('Flow', 'F')
                          .replace('Sequence', 'Seq')
                          .replace('Mapping', 'Map'))
            if token_type in ('StreamStart', 'StreamEnd'):
                continue
            output += '{:>9} {}\n'.format(token_type,
                                          self.format_stack(context['stack']))
        return output

    def test_simple_mapping(self):
        self.assertMultiLineEqual(
            self.full_stack('key: val\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:5\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('     key: val\n'),
            'BMapStart B_MAP:5\n'
            '      Key B_MAP:5 KEY:5\n'
            '   Scalar B_MAP:5 KEY:5\n'
            '    Value B_MAP:5 KEY:5 VAL:10\n'
            '   Scalar B_MAP:5\n'
            '     BEnd \n')

    def test_simple_sequence(self):
        self.assertMultiLineEqual(
            self.full_stack('- 1\n'
                            '- 2\n'
                            '- 3\n'),
            'BSeqStart B_SEQ:0\n'
            '   BEntry B_SEQ:0 B_ENT:2\n'
            '   Scalar B_SEQ:0\n'
            '   BEntry B_SEQ:0 B_ENT:2\n'
            '   Scalar B_SEQ:0\n'
            '   BEntry B_SEQ:0 B_ENT:2\n'
            '   Scalar B_SEQ:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('key:\n'
                            '  - 1\n'
                            '  - 2\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            'BSeqStart B_MAP:0 KEY:0 VAL:2 B_SEQ:2\n'
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:2 B_ENT:4\n'
            '   Scalar B_MAP:0 KEY:0 VAL:2 B_SEQ:2\n'
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:2 B_ENT:4\n'
            '   Scalar B_MAP:0 KEY:0 VAL:2 B_SEQ:2\n'
            '     BEnd B_MAP:0\n'
            '     BEnd \n')

    def test_non_indented_sequences(self):
        # There seems to be a bug in pyyaml: depending on the indentation, a
        # sequence does not produce the same tokens. More precisely, the
        # following YAML:
        #     usr:
        #       - lib
        # produces a BlockSequenceStartToken and a BlockEndToken around the
        # "lib" sequence, whereas the following:
        #     usr:
        #     - lib
        # does not (both two tokens are omitted).
        # So, yamllint must create fake 'B_SEQ'. This test makes sure it does.

        self.assertMultiLineEqual(
            self.full_stack('usr:\n'
                            '  - lib\n'
                            'var: cache\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            'BSeqStart B_MAP:0 KEY:0 VAL:2 B_SEQ:2\n'
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:2 B_ENT:4\n'
            '   Scalar B_MAP:0 KEY:0 VAL:2 B_SEQ:2\n'
            '     BEnd B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:5\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('usr:\n'
                            '- lib\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            # missing BSeqStart here
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            '   Scalar B_MAP:0\n'
            # missing BEnd here
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('usr:\n'
                            '- lib\n'
                            'var: cache\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            # missing BSeqStart here
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            '   Scalar B_MAP:0\n'
            # missing BEnd here
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:5\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('usr:\n'
                            '- []\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            # missing BSeqStart here
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            'FSeqStart B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 F_SEQ:3\n'
            '  FSeqEnd B_MAP:0\n'
            # missing BEnd here
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('usr:\n'
                            '- k:\n'
                            '    v\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            # missing BSeqStart here
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            'BMapStart B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_MAP:2\n'
            '      Key B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_MAP:2 KEY:2\n'
            '   Scalar B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_MAP:2 KEY:2\n'
            '    Value B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_MAP:2 KEY:2 VAL:4\n'  # noqa
            '   Scalar B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_MAP:2\n'
            '     BEnd B_MAP:0\n'
            # missing BEnd here
            '     BEnd \n')

    def test_flows(self):
        self.assertMultiLineEqual(
            self.full_stack('usr: [\n'
                            '  {k:\n'
                            '    v}\n'
                            '  ]\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:5\n'
            'FSeqStart B_MAP:0 KEY:0 VAL:5 F_SEQ:2\n'
            'FMapStart B_MAP:0 KEY:0 VAL:5 F_SEQ:2 F_MAP:3\n'
            '      Key B_MAP:0 KEY:0 VAL:5 F_SEQ:2 F_MAP:3 KEY:3\n'
            '   Scalar B_MAP:0 KEY:0 VAL:5 F_SEQ:2 F_MAP:3 KEY:3\n'
            '    Value B_MAP:0 KEY:0 VAL:5 F_SEQ:2 F_MAP:3 KEY:3 VAL:5\n'
            '   Scalar B_MAP:0 KEY:0 VAL:5 F_SEQ:2 F_MAP:3\n'
            '  FMapEnd B_MAP:0 KEY:0 VAL:5 F_SEQ:2\n'
            '  FSeqEnd B_MAP:0\n'
            '     BEnd \n')

    def test_anchors(self):
        self.assertMultiLineEqual(
            self.full_stack('key: &anchor value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:5\n'
            '   Anchor B_MAP:0 KEY:0 VAL:5\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('key: &anchor\n'
                            '  value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            '   Anchor B_MAP:0 KEY:0 VAL:2\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('- &anchor value\n'),
            'BSeqStart B_SEQ:0\n'
            '   BEntry B_SEQ:0 B_ENT:2\n'
            '   Anchor B_SEQ:0 B_ENT:2\n'
            '   Scalar B_SEQ:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('- &anchor\n'
                            '  value\n'),
            'BSeqStart B_SEQ:0\n'
            '   BEntry B_SEQ:0 B_ENT:2\n'
            '   Anchor B_SEQ:0 B_ENT:2\n'
            '   Scalar B_SEQ:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('- &anchor\n'
                            '  - 1\n'
                            '  - 2\n'),
            'BSeqStart B_SEQ:0\n'
            '   BEntry B_SEQ:0 B_ENT:2\n'
            '   Anchor B_SEQ:0 B_ENT:2\n'
            'BSeqStart B_SEQ:0 B_ENT:2 B_SEQ:2\n'
            '   BEntry B_SEQ:0 B_ENT:2 B_SEQ:2 B_ENT:4\n'
            '   Scalar B_SEQ:0 B_ENT:2 B_SEQ:2\n'
            '   BEntry B_SEQ:0 B_ENT:2 B_SEQ:2 B_ENT:4\n'
            '   Scalar B_SEQ:0 B_ENT:2 B_SEQ:2\n'
            '     BEnd B_SEQ:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('&anchor key:\n'
                            '  value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Anchor B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('pre:\n'
                            '  &anchor1 0\n'
                            '&anchor2 key:\n'
                            '  value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            '   Anchor B_MAP:0 KEY:0 VAL:2\n'
            '   Scalar B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Anchor B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('sequence: &anchor\n'
                            '- entry\n'
                            '- &anchor\n'
                            '  - nested\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            '   Anchor B_MAP:0 KEY:0 VAL:2\n'
            # missing BSeqStart here
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            '   Scalar B_MAP:0 KEY:0 VAL:2 B_SEQ:0\n'
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            '   Anchor B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            'BSeqStart B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_SEQ:2\n'
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_SEQ:2 B_ENT:4\n'
            '   Scalar B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_SEQ:2\n'
            '     BEnd B_MAP:0\n'
            # missing BEnd here
            '     BEnd \n')

    def test_tags(self):
        self.assertMultiLineEqual(
            self.full_stack('key: !!tag value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:5\n'
            '      Tag B_MAP:0 KEY:0 VAL:5\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('- !!map # Block collection\n'
                            '  foo : bar\n'),
            'BSeqStart B_SEQ:0\n'
            '   BEntry B_SEQ:0 B_ENT:2\n'
            '      Tag B_SEQ:0 B_ENT:2\n'
            'BMapStart B_SEQ:0 B_ENT:2 B_MAP:2\n'
            '      Key B_SEQ:0 B_ENT:2 B_MAP:2 KEY:2\n'
            '   Scalar B_SEQ:0 B_ENT:2 B_MAP:2 KEY:2\n'
            '    Value B_SEQ:0 B_ENT:2 B_MAP:2 KEY:2 VAL:8\n'
            '   Scalar B_SEQ:0 B_ENT:2 B_MAP:2\n'
            '     BEnd B_SEQ:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('- !!seq\n'
                            '  - nested item\n'),
            'BSeqStart B_SEQ:0\n'
            '   BEntry B_SEQ:0 B_ENT:2\n'
            '      Tag B_SEQ:0 B_ENT:2\n'
            'BSeqStart B_SEQ:0 B_ENT:2 B_SEQ:2\n'
            '   BEntry B_SEQ:0 B_ENT:2 B_SEQ:2 B_ENT:4\n'
            '   Scalar B_SEQ:0 B_ENT:2 B_SEQ:2\n'
            '     BEnd B_SEQ:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('sequence: !!seq\n'
                            '- entry\n'
                            '- !!seq\n'
                            '  - nested\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            '   Scalar B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:2\n'
            '      Tag B_MAP:0 KEY:0 VAL:2\n'
            # missing BSeqStart here
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            '   Scalar B_MAP:0 KEY:0 VAL:2 B_SEQ:0\n'
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            '      Tag B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2\n'
            'BSeqStart B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_SEQ:2\n'
            '   BEntry B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_SEQ:2 B_ENT:4\n'
            '   Scalar B_MAP:0 KEY:0 VAL:2 B_SEQ:0 B_ENT:2 B_SEQ:2\n'
            '     BEnd B_MAP:0\n'
            # missing BEnd here
            '     BEnd \n')

    def test_flows_imbrication(self):
        self.assertMultiLineEqual(
            self.full_stack('[[val]]\n'),
            'FSeqStart F_SEQ:1\n'
            'FSeqStart F_SEQ:1 F_SEQ:2\n'
            '   Scalar F_SEQ:1 F_SEQ:2\n'
            '  FSeqEnd F_SEQ:1\n'
            '  FSeqEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('[[val], [val2]]\n'),
            'FSeqStart F_SEQ:1\n'
            'FSeqStart F_SEQ:1 F_SEQ:2\n'
            '   Scalar F_SEQ:1 F_SEQ:2\n'
            '  FSeqEnd F_SEQ:1\n'
            '   FEntry F_SEQ:1\n'
            'FSeqStart F_SEQ:1 F_SEQ:9\n'
            '   Scalar F_SEQ:1 F_SEQ:9\n'
            '  FSeqEnd F_SEQ:1\n'
            '  FSeqEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('{{key}}\n'),
            'FMapStart F_MAP:1\n'
            'FMapStart F_MAP:1 F_MAP:2\n'
            '   Scalar F_MAP:1 F_MAP:2\n'
            '  FMapEnd F_MAP:1\n'
            '  FMapEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('[key]: value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            'FSeqStart B_MAP:0 KEY:0 F_SEQ:1\n'
            '   Scalar B_MAP:0 KEY:0 F_SEQ:1\n'
            '  FSeqEnd B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:7\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('[[key]]: value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            'FSeqStart B_MAP:0 KEY:0 F_SEQ:1\n'
            'FSeqStart B_MAP:0 KEY:0 F_SEQ:1 F_SEQ:2\n'
            '   Scalar B_MAP:0 KEY:0 F_SEQ:1 F_SEQ:2\n'
            '  FSeqEnd B_MAP:0 KEY:0 F_SEQ:1\n'
            '  FSeqEnd B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:9\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('{key}: value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            'FMapStart B_MAP:0 KEY:0 F_MAP:1\n'
            '   Scalar B_MAP:0 KEY:0 F_MAP:1\n'
            '  FMapEnd B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:7\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('{key: value}: value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            'FMapStart B_MAP:0 KEY:0 F_MAP:1\n'
            '      Key B_MAP:0 KEY:0 F_MAP:1 KEY:1\n'
            '   Scalar B_MAP:0 KEY:0 F_MAP:1 KEY:1\n'
            '    Value B_MAP:0 KEY:0 F_MAP:1 KEY:1 VAL:6\n'
            '   Scalar B_MAP:0 KEY:0 F_MAP:1\n'
            '  FMapEnd B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:14\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('{{key}}: value\n'),
            'BMapStart B_MAP:0\n'
            '      Key B_MAP:0 KEY:0\n'
            'FMapStart B_MAP:0 KEY:0 F_MAP:1\n'
            'FMapStart B_MAP:0 KEY:0 F_MAP:1 F_MAP:2\n'
            '   Scalar B_MAP:0 KEY:0 F_MAP:1 F_MAP:2\n'
            '  FMapEnd B_MAP:0 KEY:0 F_MAP:1\n'
            '  FMapEnd B_MAP:0 KEY:0\n'
            '    Value B_MAP:0 KEY:0 VAL:9\n'
            '   Scalar B_MAP:0\n'
            '     BEnd \n')
        self.assertMultiLineEqual(
            self.full_stack('{{key}: val, {key2}: {val2}}\n'),
            'FMapStart F_MAP:1\n'
            '      Key F_MAP:1 KEY:1\n'
            'FMapStart F_MAP:1 KEY:1 F_MAP:2\n'
            '   Scalar F_MAP:1 KEY:1 F_MAP:2\n'
            '  FMapEnd F_MAP:1 KEY:1\n'
            '    Value F_MAP:1 KEY:1 VAL:8\n'
            '   Scalar F_MAP:1\n'
            '   FEntry F_MAP:1\n'
            '      Key F_MAP:1 KEY:1\n'
            'FMapStart F_MAP:1 KEY:1 F_MAP:14\n'
            '   Scalar F_MAP:1 KEY:1 F_MAP:14\n'
            '  FMapEnd F_MAP:1 KEY:1\n'
            '    Value F_MAP:1 KEY:1 VAL:21\n'
            'FMapStart F_MAP:1 KEY:1 VAL:21 F_MAP:22\n'
            '   Scalar F_MAP:1 KEY:1 VAL:21 F_MAP:22\n'
            '  FMapEnd F_MAP:1\n'
            '  FMapEnd \n')

        self.assertMultiLineEqual(
            self.full_stack('{[{{[val]}}, [{[key]: val2}]]}\n'),
            'FMapStart F_MAP:1\n'
            'FSeqStart F_MAP:1 F_SEQ:2\n'
            'FMapStart F_MAP:1 F_SEQ:2 F_MAP:3\n'
            'FMapStart F_MAP:1 F_SEQ:2 F_MAP:3 F_MAP:4\n'
            'FSeqStart F_MAP:1 F_SEQ:2 F_MAP:3 F_MAP:4 F_SEQ:5\n'
            '   Scalar F_MAP:1 F_SEQ:2 F_MAP:3 F_MAP:4 F_SEQ:5\n'
            '  FSeqEnd F_MAP:1 F_SEQ:2 F_MAP:3 F_MAP:4\n'
            '  FMapEnd F_MAP:1 F_SEQ:2 F_MAP:3\n'
            '  FMapEnd F_MAP:1 F_SEQ:2\n'
            '   FEntry F_MAP:1 F_SEQ:2\n'
            'FSeqStart F_MAP:1 F_SEQ:2 F_SEQ:14\n'
            'FMapStart F_MAP:1 F_SEQ:2 F_SEQ:14 F_MAP:15\n'
            '      Key F_MAP:1 F_SEQ:2 F_SEQ:14 F_MAP:15 KEY:15\n'
            'FSeqStart F_MAP:1 F_SEQ:2 F_SEQ:14 F_MAP:15 KEY:15 F_SEQ:16\n'
            '   Scalar F_MAP:1 F_SEQ:2 F_SEQ:14 F_MAP:15 KEY:15 F_SEQ:16\n'
            '  FSeqEnd F_MAP:1 F_SEQ:2 F_SEQ:14 F_MAP:15 KEY:15\n'
            '    Value F_MAP:1 F_SEQ:2 F_SEQ:14 F_MAP:15 KEY:15 VAL:22\n'
            '   Scalar F_MAP:1 F_SEQ:2 F_SEQ:14 F_MAP:15\n'
            '  FMapEnd F_MAP:1 F_SEQ:2 F_SEQ:14\n'
            '  FSeqEnd F_MAP:1 F_SEQ:2\n'
            '  FSeqEnd F_MAP:1\n'
            '  FMapEnd \n')


class IndentationTestCase(RuleTestCase):
    rule_id = 'indentation'

    def test_disabled(self):
        conf = 'indentation: disable'
        self.check('---\n'
                   'object:\n'
                   '   k1: v1\n'
                   'obj2:\n'
                   ' k2:\n'
                   '     - 8\n'
                   ' k3:\n'
                   '           val\n'
                   '...\n', conf)
        self.check('---\n'
                   '  o:\n'
                   '    k1: v1\n'
                   '  p:\n'
                   '   k3:\n'
                   '       val\n'
                   '...\n', conf)
        self.check('---\n'
                   '     - o:\n'
                   '         k1: v1\n'
                   '     - p: kdjf\n'
                   '     - q:\n'
                   '        k3:\n'
                   '              - val\n'
                   '...\n', conf)

    def test_one_space(self):
        conf = 'indentation: {spaces: 1, indent-sequences: false}'
        self.check('---\n'
                   'object:\n'
                   ' k1:\n'
                   ' - a\n'
                   ' - b\n'
                   ' k2: v2\n'
                   ' k3:\n'
                   ' - name: Unix\n'
                   '   date: 1969\n'
                   ' - name: Linux\n'
                   '   date: 1991\n'
                   '...\n', conf)
        conf = 'indentation: {spaces: 1, indent-sequences: true}'
        self.check('---\n'
                   'object:\n'
                   ' k1:\n'
                   '  - a\n'
                   '  - b\n'
                   ' k2: v2\n'
                   ' k3:\n'
                   '  - name: Unix\n'
                   '    date: 1969\n'
                   '  - name: Linux\n'
                   '    date: 1991\n'
                   '...\n', conf)

    def test_two_spaces(self):
        conf = 'indentation: {spaces: 2, indent-sequences: false}'
        self.check('---\n'
                   'object:\n'
                   '  k1:\n'
                   '  - a\n'
                   '  - b\n'
                   '  k2: v2\n'
                   '  k3:\n'
                   '  - name: Unix\n'
                   '    date: 1969\n'
                   '  - name: Linux\n'
                   '    date: 1991\n'
                   '  k4:\n'
                   '  -\n'
                   '  k5: v3\n'
                   '...\n', conf)
        conf = 'indentation: {spaces: 2, indent-sequences: true}'
        self.check('---\n'
                   'object:\n'
                   '  k1:\n'
                   '    - a\n'
                   '    - b\n'
                   '  k2: v2\n'
                   '  k3:\n'
                   '    - name: Unix\n'
                   '      date: 1969\n'
                   '    - name: Linux\n'
                   '      date: 1991\n'
                   '...\n', conf)

    def test_three_spaces(self):
        conf = 'indentation: {spaces: 3, indent-sequences: false}'
        self.check('---\n'
                   'object:\n'
                   '   k1:\n'
                   '   - a\n'
                   '   - b\n'
                   '   k2: v2\n'
                   '   k3:\n'
                   '   - name: Unix\n'
                   '     date: 1969\n'
                   '   - name: Linux\n'
                   '     date: 1991\n'
                   '...\n', conf)
        conf = 'indentation: {spaces: 3, indent-sequences: true}'
        self.check('---\n'
                   'object:\n'
                   '   k1:\n'
                   '      - a\n'
                   '      - b\n'
                   '   k2: v2\n'
                   '   k3:\n'
                   '      - name: Unix\n'
                   '        date: 1969\n'
                   '      - name: Linux\n'
                   '        date: 1991\n'
                   '...\n', conf)

    def test_consistent_spaces(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              indent-sequences: whatever}\n'
                'document-start: disable\n')
        self.check('---\n'
                   'object:\n'
                   ' k1:\n'
                   '  - a\n'
                   '  - b\n'
                   ' k2: v2\n'
                   ' k3:\n'
                   '  - name: Unix\n'
                   '    date: 1969\n'
                   '  - name: Linux\n'
                   '    date: 1991\n'
                   '...\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  k1:\n'
                   '  - a\n'
                   '  - b\n'
                   '  k2: v2\n'
                   '  k3:\n'
                   '  - name: Unix\n'
                   '    date: 1969\n'
                   '  - name: Linux\n'
                   '    date: 1991\n'
                   '...\n', conf)
        self.check('---\n'
                   'object:\n'
                   '   k1:\n'
                   '      - a\n'
                   '      - b\n'
                   '   k2: v2\n'
                   '   k3:\n'
                   '      - name: Unix\n'
                   '        date: 1969\n'
                   '      - name: Linux\n'
                   '        date: 1991\n'
                   '...\n', conf)
        self.check('first is not indented:\n'
                   '  value is indented\n', conf)
        self.check('first is not indented:\n'
                   '     value:\n'
                   '          is indented\n', conf)
        self.check('- first is already indented:\n'
                   '    value is indented too\n', conf)
        self.check('- first is already indented:\n'
                   '       value:\n'
                   '            is indented too\n', conf)
        self.check('- first is already indented:\n'
                   '       value:\n'
                   '             is indented too\n', conf, problem=(3, 14))
        self.check('---\n'
                   'list one:\n'
                   '  - 1\n'
                   '  - 2\n'
                   '  - 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf, problem=(7, 5))
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '  - a\n'
                   '  - b\n'
                   '  - c\n', conf)
        self.check('---\n'
                   'list one:\n'
                   ' - 1\n'
                   ' - 2\n'
                   ' - 3\n'
                   'list two:\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf)

    def test_consistent_spaces_and_indent_sequences(self):
        conf = 'indentation: {spaces: consistent, indent-sequences: true}'
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf, problem1=(3, 1))
        self.check('---\n'
                   'list one:\n'
                   '  - 1\n'
                   '  - 2\n'
                   '  - 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf, problem1=(7, 5))
        self.check('---\n'
                   'list one:\n'
                   '  - 1\n'
                   '  - 2\n'
                   '  - 3\n'
                   'list two:\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf, problem1=(7, 1))

        conf = 'indentation: {spaces: consistent, indent-sequences: false}'
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf, problem1=(7, 5))
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '  - a\n'
                   '  - b\n'
                   '  - c\n', conf, problem1=(7, 3))
        self.check('---\n'
                   'list one:\n'
                   '  - 1\n'
                   '  - 2\n'
                   '  - 3\n'
                   'list two:\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf, problem1=(3, 3))

        conf = ('indentation: {spaces: consistent,\n'
                '              indent-sequences: consistent}')
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf, problem1=(7, 5))
        self.check('---\n'
                   'list one:\n'
                   '    - 1\n'
                   '    - 2\n'
                   '    - 3\n'
                   'list two:\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf, problem1=(7, 1))
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf)
        self.check('---\n'
                   'list one:\n'
                   '  - 1\n'
                   '  - 2\n'
                   '  - 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf, problem1=(7, 5))

        conf = 'indentation: {spaces: consistent, indent-sequences: whatever}'
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf)
        self.check('---\n'
                   'list one:\n'
                   '    - 1\n'
                   '    - 2\n'
                   '    - 3\n'
                   'list two:\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf)
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf)
        self.check('---\n'
                   'list one:\n'
                   '  - 1\n'
                   '  - 2\n'
                   '  - 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf, problem1=(7, 5))

    def test_indent_sequences_whatever(self):
        conf = 'indentation: {spaces: 4, indent-sequences: whatever}'
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf)
        self.check('---\n'
                   'list one:\n'
                   '  - 1\n'
                   '  - 2\n'
                   '  - 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf, problem=(3, 3))
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '  - a\n'
                   '  - b\n'
                   '  - c\n', conf, problem=(7, 3))
        self.check('---\n'
                   'list:\n'
                   '    - 1\n'
                   '    - 2\n'
                   '    - 3\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf, problem=(6, 1, 'syntax'))

    def test_indent_sequences_consistent(self):
        conf = 'indentation: {spaces: 4, indent-sequences: consistent}'
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list:\n'
                   '    two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf)
        self.check('---\n'
                   'list one:\n'
                   '    - 1\n'
                   '    - 2\n'
                   '    - 3\n'
                   'list:\n'
                   '    two:\n'
                   '        - a\n'
                   '        - b\n'
                   '        - c\n', conf)
        self.check('---\n'
                   'list one:\n'
                   '- 1\n'
                   '- 2\n'
                   '- 3\n'
                   'list two:\n'
                   '    - a\n'
                   '    - b\n'
                   '    - c\n', conf, problem=(7, 5))
        self.check('---\n'
                   'list one:\n'
                   '    - 1\n'
                   '    - 2\n'
                   '    - 3\n'
                   'list two:\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf, problem=(7, 1))
        self.check('---\n'
                   'list one:\n'
                   ' - 1\n'
                   ' - 2\n'
                   ' - 3\n'
                   'list two:\n'
                   '- a\n'
                   '- b\n'
                   '- c\n', conf, problem1=(3, 2), problem2=(7, 1))

    def test_direct_flows(self):
        # flow: [ ...
        # ]
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   'a: {x: 1,\n'
                   '    y,\n'
                   '    z: 1}\n', conf)
        self.check('---\n'
                   'a: {x: 1,\n'
                   '   y,\n'
                   '    z: 1}\n', conf, problem=(3, 4))
        self.check('---\n'
                   'a: {x: 1,\n'
                   '     y,\n'
                   '    z: 1}\n', conf, problem=(3, 6))
        self.check('---\n'
                   'a: {x: 1,\n'
                   '  y, z: 1}\n', conf, problem=(3, 3))
        self.check('---\n'
                   'a: {x: 1,\n'
                   '    y, z: 1\n'
                   '}\n', conf)
        self.check('---\n'
                   'a: {x: 1,\n'
                   '  y, z: 1\n'
                   '}\n', conf, problem=(3, 3))
        self.check('---\n'
                   'a: [x,\n'
                   '    y,\n'
                   '    z]\n', conf)
        self.check('---\n'
                   'a: [x,\n'
                   '   y,\n'
                   '    z]\n', conf, problem=(3, 4))
        self.check('---\n'
                   'a: [x,\n'
                   '     y,\n'
                   '    z]\n', conf, problem=(3, 6))
        self.check('---\n'
                   'a: [x,\n'
                   '  y, z]\n', conf, problem=(3, 3))
        self.check('---\n'
                   'a: [x,\n'
                   '    y, z\n'
                   ']\n', conf)
        self.check('---\n'
                   'a: [x,\n'
                   '  y, z\n'
                   ']\n', conf, problem=(3, 3))

    def test_broken_flows(self):
        # flow: [
        #   ...
        # ]
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   'a: {\n'
                   '  x: 1,\n'
                   '  y, z: 1\n'
                   '}\n', conf)
        self.check('---\n'
                   'a: {\n'
                   '  x: 1,\n'
                   '  y, z: 1}\n', conf)
        self.check('---\n'
                   'a: {\n'
                   '   x: 1,\n'
                   '  y, z: 1\n'
                   '}\n', conf, problem=(4, 3))
        self.check('---\n'
                   'a: {\n'
                   '  x: 1,\n'
                   '  y, z: 1\n'
                   '  }\n', conf, problem=(5, 3))
        self.check('---\n'
                   'a: [\n'
                   '  x,\n'
                   '  y, z\n'
                   ']\n', conf)
        self.check('---\n'
                   'a: [\n'
                   '  x,\n'
                   '  y, z]\n', conf)
        self.check('---\n'
                   'a: [\n'
                   '   x,\n'
                   '  y, z\n'
                   ']\n', conf, problem=(4, 3))
        self.check('---\n'
                   'a: [\n'
                   '  x,\n'
                   '  y, z\n'
                   '  ]\n', conf, problem=(5, 3))
        self.check('---\n'
                   'obj: {\n'
                   '  a: 1,\n'
                   '   b: 2,\n'
                   ' c: 3\n'
                   '}\n', conf, problem1=(4, 4), problem2=(5, 2))
        self.check('---\n'
                   'list: [\n'
                   '  1,\n'
                   '   2,\n'
                   ' 3\n'
                   ']\n', conf, problem1=(4, 4), problem2=(5, 2))
        self.check('---\n'
                   'top:\n'
                   '  rules: [\n'
                   '    1, 2,\n'
                   '  ]\n', conf)
        self.check('---\n'
                   'top:\n'
                   '  rules: [\n'
                   '    1, 2,\n'
                   ']\n'
                   '  rulez: [\n'
                   '    1, 2,\n'
                   '    ]\n', conf, problem1=(5, 1), problem2=(8, 5))
        self.check('---\n'
                   'top:\n'
                   '  rules:\n'
                   '    here: {\n'
                   '      foo: 1,\n'
                   '      bar: 2\n'
                   '    }\n', conf)
        self.check('---\n'
                   'top:\n'
                   '  rules:\n'
                   '    here: {\n'
                   '      foo: 1,\n'
                   '      bar: 2\n'
                   '      }\n'
                   '    there: {\n'
                   '      foo: 1,\n'
                   '      bar: 2\n'
                   '  }\n', conf, problem1=(7, 7), problem2=(11, 3))
        conf = 'indentation: {spaces: 2}'
        self.check('---\n'
                   'a: {\n'
                   '   x: 1,\n'
                   '  y, z: 1\n'
                   '}\n', conf, problem=(3, 4))
        self.check('---\n'
                   'a: [\n'
                   '   x,\n'
                   '  y, z\n'
                   ']\n', conf, problem=(3, 4))

    def test_cleared_flows(self):
        # flow:
        #   [
        #     ...
        #   ]
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   'top:\n'
                   '  rules:\n'
                   '    {\n'
                   '      foo: 1,\n'
                   '      bar: 2\n'
                   '    }\n', conf)
        self.check('---\n'
                   'top:\n'
                   '  rules:\n'
                   '    {\n'
                   '       foo: 1,\n'
                   '      bar: 2\n'
                   '    }\n', conf, problem=(5, 8))
        self.check('---\n'
                   'top:\n'
                   '  rules:\n'
                   '   {\n'
                   '     foo: 1,\n'
                   '     bar: 2\n'
                   '   }\n', conf, problem=(4, 4))
        self.check('---\n'
                   'top:\n'
                   '  rules:\n'
                   '    {\n'
                   '      foo: 1,\n'
                   '      bar: 2\n'
                   '   }\n', conf, problem=(7, 4))
        self.check('---\n'
                   'top:\n'
                   '  rules:\n'
                   '    {\n'
                   '      foo: 1,\n'
                   '      bar: 2\n'
                   '     }\n', conf, problem=(7, 6))
        self.check('---\n'
                   'top:\n'
                   '  [\n'
                   '    a, b, c\n'
                   '  ]\n', conf)
        self.check('---\n'
                   'top:\n'
                   '  [\n'
                   '     a, b, c\n'
                   '  ]\n', conf, problem=(4, 6))
        self.check('---\n'
                   'top:\n'
                   '   [\n'
                   '     a, b, c\n'
                   '   ]\n', conf, problem=(4, 6))
        self.check('---\n'
                   'top:\n'
                   '  [\n'
                   '    a, b, c\n'
                   '   ]\n', conf, problem=(5, 4))
        self.check('---\n'
                   'top:\n'
                   '  rules: [\n'
                   '    {\n'
                   '      foo: 1\n'
                   '    },\n'
                   '    {\n'
                   '      foo: 2,\n'
                   '      bar: [\n'
                   '        a, b, c\n'
                   '      ],\n'
                   '    },\n'
                   '  ]\n', conf)
        self.check('---\n'
                   'top:\n'
                   '  rules: [\n'
                   '    {\n'
                   '     foo: 1\n'
                   '     },\n'
                   '    {\n'
                   '      foo: 2,\n'
                   '        bar: [\n'
                   '          a, b, c\n'
                   '      ],\n'
                   '    },\n'
                   ']\n', conf, problem1=(5, 6), problem2=(6, 6),
                   problem3=(9, 9), problem4=(11, 7), problem5=(13, 1))

    def test_under_indented(self):
        conf = 'indentation: {spaces: 2, indent-sequences: consistent}'
        self.check('---\n'
                   'object:\n'
                   ' val: 1\n'
                   '...\n', conf, problem=(3, 2))
        self.check('---\n'
                   'object:\n'
                   '  k1:\n'
                   '   - a\n'
                   '...\n', conf, problem=(4, 4))
        self.check('---\n'
                   'object:\n'
                   '  k3:\n'
                   '    - name: Unix\n'
                   '     date: 1969\n'
                   '...\n', conf, problem=(5, 6, 'syntax'))
        conf = 'indentation: {spaces: 4, indent-sequences: consistent}'
        self.check('---\n'
                   'object:\n'
                   '   val: 1\n'
                   '...\n', conf, problem=(3, 4))
        self.check('---\n'
                   '- el1\n'
                   '- el2:\n'
                   '   - subel\n'
                   '...\n', conf, problem=(4, 4))
        self.check('---\n'
                   'object:\n'
                   '    k3:\n'
                   '        - name: Linux\n'
                   '         date: 1991\n'
                   '...\n', conf, problem=(5, 10, 'syntax'))
        conf = 'indentation: {spaces: 2, indent-sequences: true}'
        self.check('---\n'
                   'a:\n'
                   '-\n'  # empty list
                   'b: c\n'
                   '...\n', conf, problem=(3, 1))
        conf = 'indentation: {spaces: 2, indent-sequences: consistent}'
        self.check('---\n'
                   'a:\n'
                   '  -\n'  # empty list
                   'b:\n'
                   '-\n'
                   'c: d\n'
                   '...\n', conf, problem=(5, 1))

    def test_over_indented(self):
        conf = 'indentation: {spaces: 2, indent-sequences: consistent}'
        self.check('---\n'
                   'object:\n'
                   '   val: 1\n'
                   '...\n', conf, problem=(3, 4))
        self.check('---\n'
                   'object:\n'
                   '  k1:\n'
                   '     - a\n'
                   '...\n', conf, problem=(4, 6))
        self.check('---\n'
                   'object:\n'
                   '  k3:\n'
                   '    - name: Unix\n'
                   '       date: 1969\n'
                   '...\n', conf, problem=(5, 12, 'syntax'))
        conf = 'indentation: {spaces: 4, indent-sequences: consistent}'
        self.check('---\n'
                   'object:\n'
                   '     val: 1\n'
                   '...\n', conf, problem=(3, 6))
        self.check('---\n'
                   ' object:\n'
                   '     val: 1\n'
                   '...\n', conf, problem=(2, 2))
        self.check('---\n'
                   '- el1\n'
                   '- el2:\n'
                   '     - subel\n'
                   '...\n', conf, problem=(4, 6))
        self.check('---\n'
                   '- el1\n'
                   '- el2:\n'
                   '              - subel\n'
                   '...\n', conf, problem=(4, 15))
        self.check('---\n'
                   '  - el1\n'
                   '  - el2:\n'
                   '        - subel\n'
                   '...\n', conf,
                   problem=(2, 3))
        self.check('---\n'
                   'object:\n'
                   '    k3:\n'
                   '        - name: Linux\n'
                   '           date: 1991\n'
                   '...\n', conf, problem=(5, 16, 'syntax'))
        conf = 'indentation: {spaces: 4, indent-sequences: whatever}'
        self.check('---\n'
                   '  - el1\n'
                   '  - el2:\n'
                   '    - subel\n'
                   '...\n', conf,
                   problem=(2, 3))
        conf = 'indentation: {spaces: 2, indent-sequences: false}'
        self.check('---\n'
                   'a:\n'
                   '  -\n'  # empty list
                   'b: c\n'
                   '...\n', conf, problem=(3, 3))
        conf = 'indentation: {spaces: 2, indent-sequences: consistent}'
        self.check('---\n'
                   'a:\n'
                   '-\n'  # empty list
                   'b:\n'
                   '  -\n'
                   'c: d\n'
                   '...\n', conf, problem=(5, 3))

    def test_multi_lines(self):
        conf = 'indentation: {spaces: consistent, indent-sequences: true}'
        self.check('---\n'
                   'long_string: >\n'
                   '  bla bla blah\n'
                   '  blah bla bla\n'
                   '...\n', conf)
        self.check('---\n'
                   '- long_string: >\n'
                   '    bla bla blah\n'
                   '    blah bla bla\n'
                   '...\n', conf)
        self.check('---\n'
                   'obj:\n'
                   '  - long_string: >\n'
                   '      bla bla blah\n'
                   '      blah bla bla\n'
                   '...\n', conf)

    def test_empty_value(self):
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   'key1:\n'
                   'key2: not empty\n'
                   'key3:\n'
                   '...\n', conf)
        self.check('---\n'
                   '-\n'
                   '- item 2\n'
                   '-\n'
                   '...\n', conf)

    def test_nested_collections(self):
        conf = 'indentation: {spaces: 2}'
        self.check('---\n'
                   '- o:\n'
                   '  k1: v1\n'
                   '...\n', conf)
        self.check('---\n'
                   '- o:\n'
                   ' k1: v1\n'
                   '...\n', conf, problem=(3, 2, 'syntax'))
        self.check('---\n'
                   '- o:\n'
                   '   k1: v1\n'
                   '...\n', conf, problem=(3, 4))
        conf = 'indentation: {spaces: 4}'
        self.check('---\n'
                   '- o:\n'
                   '      k1: v1\n'
                   '...\n', conf)
        self.check('---\n'
                   '- o:\n'
                   '     k1: v1\n'
                   '...\n', conf, problem=(3, 6))
        self.check('---\n'
                   '- o:\n'
                   '       k1: v1\n'
                   '...\n', conf, problem=(3, 8))
        self.check('---\n'
                   '- - - - item\n'
                   '    - elem 1\n'
                   '    - elem 2\n'
                   '    - - - - - very nested: a\n'
                   '              key: value\n'
                   '...\n', conf)
        self.check('---\n'
                   ' - - - - item\n'
                   '     - elem 1\n'
                   '     - elem 2\n'
                   '     - - - - - very nested: a\n'
                   '               key: value\n'
                   '...\n', conf, problem=(2, 2))

    def test_return(self):
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   'a:\n'
                   '  b:\n'
                   '    c:\n'
                   '  d:\n'
                   '    e:\n'
                   '      f:\n'
                   'g:\n'
                   '...\n', conf)
        self.check('---\n'
                   'a:\n'
                   '  b:\n'
                   '    c:\n'
                   '   d:\n'
                   '...\n', conf, problem=(5, 4, 'syntax'))
        self.check('---\n'
                   'a:\n'
                   '  b:\n'
                   '    c:\n'
                   ' d:\n'
                   '...\n', conf, problem=(5, 2, 'syntax'))

    def test_first_line(self):
        conf = ('indentation: {spaces: consistent}\n'
                'document-start: disable\n')
        self.check('  a: 1\n', conf, problem=(1, 3))

    def test_explicit_block_mappings(self):
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   'object:\n'
                   '    ? key\n'
                   '    : value\n', conf)
        self.check('---\n'
                   'object:\n'
                   '    ? key\n'
                   '    :\n'
                   '        value\n'
                   '...\n', conf)
        self.check('---\n'
                   'object:\n'
                   '    ?\n'
                   '        key\n'
                   '    : value\n', conf)
        self.check('---\n'
                   'object:\n'
                   '    ?\n'
                   '        key\n'
                   '    :\n'
                   '        value\n'
                   '...\n', conf)
        self.check('---\n'
                   '- ? key\n'
                   '  : value\n', conf)
        self.check('---\n'
                   '- ? key\n'
                   '  :\n'
                   '      value\n'
                   '...\n', conf)
        self.check('---\n'
                   '- ?\n'
                   '      key\n'
                   '  : value\n', conf)
        self.check('---\n'
                   '- ?\n'
                   '      key\n'
                   '  :\n'
                   '      value\n'
                   '...\n', conf)
        self.check('---\n'
                   'object:\n'
                   '    ? key\n'
                   '    :\n'
                   '       value\n'
                   '...\n', conf, problem=(5, 8))
        self.check('---\n'
                   '- - ?\n'
                   '       key\n'
                   '    :\n'
                   '      value\n'
                   '...\n', conf, problem=(5, 7))
        self.check('---\n'
                   'object:\n'
                   '    ?\n'
                   '       key\n'
                   '    :\n'
                   '         value\n'
                   '...\n', conf, problem1=(4, 8), problem2=(6, 10))
        self.check('---\n'
                   'object:\n'
                   '    ?\n'
                   '         key\n'
                   '    :\n'
                   '       value\n'
                   '...\n', conf, problem1=(4, 10), problem2=(6, 8))

    def test_clear_sequence_item(self):
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   '-\n'
                   '  string\n'
                   '-\n'
                   '  map: ping\n'
                   '-\n'
                   '  - sequence\n'
                   '  -\n'
                   '    nested\n'
                   '  -\n'
                   '    >\n'
                   '      multi\n'
                   '      line\n'
                   '...\n', conf)
        self.check('---\n'
                   '-\n'
                   ' string\n'
                   '-\n'
                   '   string\n', conf, problem=(5, 4))
        self.check('---\n'
                   '-\n'
                   ' map: ping\n'
                   '-\n'
                   '   map: ping\n', conf, problem=(5, 4))
        self.check('---\n'
                   '-\n'
                   ' - sequence\n'
                   '-\n'
                   '   - sequence\n', conf, problem=(5, 4))
        self.check('---\n'
                   '-\n'
                   '  -\n'
                   '   nested\n'
                   '  -\n'
                   '     nested\n', conf, problem1=(4, 4), problem2=(6, 6))
        self.check('---\n'
                   '-\n'
                   '  -\n'
                   '     >\n'
                   '      multi\n'
                   '      line\n'
                   '...\n', conf, problem=(4, 6))
        conf = 'indentation: {spaces: 2}'
        self.check('---\n'
                   '-\n'
                   ' string\n'
                   '-\n'
                   '   string\n', conf, problem1=(3, 2), problem2=(5, 4))
        self.check('---\n'
                   '-\n'
                   ' map: ping\n'
                   '-\n'
                   '   map: ping\n', conf, problem1=(3, 2), problem2=(5, 4))
        self.check('---\n'
                   '-\n'
                   ' - sequence\n'
                   '-\n'
                   '   - sequence\n', conf, problem1=(3, 2), problem2=(5, 4))
        self.check('---\n'
                   '-\n'
                   '  -\n'
                   '   nested\n'
                   '  -\n'
                   '     nested\n', conf, problem1=(4, 4), problem2=(6, 6))

    def test_anchors(self):
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   'key: &anchor value\n', conf)
        self.check('---\n'
                   'key: &anchor\n'
                   '  value\n', conf)
        self.check('---\n'
                   '- &anchor value\n', conf)
        self.check('---\n'
                   '- &anchor\n'
                   '  value\n', conf)
        self.check('---\n'
                   'key: &anchor [1,\n'
                   '              2]\n', conf)
        self.check('---\n'
                   'key: &anchor\n'
                   '  [1,\n'
                   '   2]\n', conf)
        self.check('---\n'
                   'key: &anchor\n'
                   '  - 1\n'
                   '  - 2\n', conf)
        self.check('---\n'
                   '- &anchor [1,\n'
                   '           2]\n', conf)
        self.check('---\n'
                   '- &anchor\n'
                   '  [1,\n'
                   '   2]\n', conf)
        self.check('---\n'
                   '- &anchor\n'
                   '  - 1\n'
                   '  - 2\n', conf)
        self.check('---\n'
                   'key:\n'
                   '  &anchor1\n'
                   '  value\n', conf)
        self.check('---\n'
                   'pre:\n'
                   '  &anchor1 0\n'
                   '&anchor2 key:\n'
                   '  value\n', conf)
        self.check('---\n'
                   'machine0:\n'
                   '  /etc/hosts: &ref-etc-hosts\n'
                   '    content:\n'
                   '      - 127.0.0.1: localhost\n'
                   '      - ::1: localhost\n'
                   '    mode: 0644\n'
                   'machine1:\n'
                   '  /etc/hosts: *ref-etc-hosts\n', conf)
        self.check('---\n'
                   'list:\n'
                   '  - k: v\n'
                   '  - &a truc\n'
                   '  - &b\n'
                   '    truc\n'
                   '  - k: *a\n', conf)

    def test_tags(self):
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   '-\n'
                   '  "flow in block"\n'
                   '- >\n'
                   '    Block scalar\n'
                   '- !!map  # Block collection\n'
                   '  foo: bar\n', conf)

        conf = 'indentation: {spaces: consistent, indent-sequences: false}'
        self.check('---\n'
                   'sequence: !!seq\n'
                   '- entry\n'
                   '- !!seq\n'
                   '  - nested\n', conf)
        self.check('---\n'
                   'mapping: !!map\n'
                   '  foo: bar\n'
                   'Block style: !!map\n'
                   '  Clark: Evans\n'
                   '  Ingy: döt Net\n'
                   '  Oren: Ben-Kiki\n', conf)
        self.check('---\n'
                   'Flow style: !!map {Clark: Evans, Ingy: döt Net}\n'
                   'Block style: !!seq\n'
                   '- Clark Evans\n'
                   '- Ingy döt Net\n', conf)

    def test_flows_imbrication(self):
        conf = 'indentation: {spaces: consistent}'
        self.check('---\n'
                   '[val]: value\n', conf)
        self.check('---\n'
                   '{key}: value\n', conf)
        self.check('---\n'
                   '{key: val}: value\n', conf)
        self.check('---\n'
                   '[[val]]: value\n', conf)
        self.check('---\n'
                   '{{key}}: value\n', conf)
        self.check('---\n'
                   '{{key: val1}: val2}: value\n', conf)
        self.check('---\n'
                   '- [val, {{key: val}: val}]: value\n'
                   '- {[val,\n'
                   '    {{key: val}: val}]}\n'
                   '- {[val,\n'
                   '    {{key: val,\n'
                   '      key2}}]}\n'
                   '- {{{{{moustaches}}}}}\n'
                   '- {{{{{moustache,\n'
                   '       moustache},\n'
                   '      moustache}},\n'
                   '    moustache}}\n', conf)
        self.check('---\n'
                   '- {[val,\n'
                   '     {{key: val}: val}]}\n',
                   conf, problem=(3, 6))
        self.check('---\n'
                   '- {[val,\n'
                   '    {{key: val,\n'
                   '     key2}}]}\n',
                   conf, problem=(4, 6))
        self.check('---\n'
                   '- {{{{{moustache,\n'
                   '       moustache},\n'
                   '       moustache}},\n'
                   '   moustache}}\n',
                   conf, problem1=(4, 8), problem2=(5, 4))


class ScalarIndentationTestCase(RuleTestCase):
    rule_id = 'indentation'

    def test_basics_plain(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: false}\n'
                'document-start: disable\n')
        self.check('multi\n'
                   'line\n', conf)
        self.check('multi\n'
                   ' line\n', conf)
        self.check('- multi\n'
                   '  line\n', conf)
        self.check('- multi\n'
                   '   line\n', conf)
        self.check('a key: multi\n'
                   '       line\n', conf)
        self.check('a key: multi\n'
                   '  line\n', conf)
        self.check('a key: multi\n'
                   '        line\n', conf)
        self.check('a key:\n'
                   '  multi\n'
                   '  line\n', conf)
        self.check('- C code: void main() {\n'
                   '              printf("foo");\n'
                   '          }\n', conf)
        self.check('- C code:\n'
                   '    void main() {\n'
                   '        printf("foo");\n'
                   '    }\n', conf)

    def test_check_multi_line_plain(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('multi\n'
                   ' line\n', conf, problem=(2, 2))
        self.check('- multi\n'
                   '   line\n', conf, problem=(2, 4))
        self.check('a key: multi\n'
                   '  line\n', conf, problem=(2, 3))
        self.check('a key: multi\n'
                   '        line\n', conf, problem=(2, 9))
        self.check('a key:\n'
                   '  multi\n'
                   '   line\n', conf, problem=(3, 4))
        self.check('- C code: void main() {\n'
                   '              printf("foo");\n'
                   '          }\n', conf, problem=(2, 15))
        self.check('- C code:\n'
                   '    void main() {\n'
                   '        printf("foo");\n'
                   '    }\n', conf, problem=(3, 9))

    def test_basics_quoted(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: false}\n'
                'document-start: disable\n')
        self.check('"multi\n'
                   ' line"\n', conf)
        self.check('- "multi\n'
                   '   line"\n', conf)
        self.check('a key: "multi\n'
                   '        line"\n', conf)
        self.check('a key:\n'
                   '  "multi\n'
                   '   line"\n', conf)
        self.check('- jinja2: "{% if ansible is defined %}\n'
                   '             {{ ansible }}\n'
                   '           {% else %}\n'
                   '             {{ chef }}\n'
                   '           {% endif %}"\n', conf)
        self.check('- jinja2:\n'
                   '    "{% if ansible is defined %}\n'
                   '       {{ ansible }}\n'
                   '     {% else %}\n'
                   '       {{ chef }}\n'
                   '     {% endif %}"\n', conf)
        self.check('["this is a very long line\n'
                   '  that needs to be split",\n'
                   ' "other line"]\n', conf)
        self.check('["multi\n'
                   '  line 1", "multi\n'
                   '            line 2"]\n', conf)

    def test_check_multi_line_quoted(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('"multi\n'
                   'line"\n', conf, problem=(2, 1))
        self.check('"multi\n'
                   '  line"\n', conf, problem=(2, 3))
        self.check('- "multi\n'
                   '  line"\n', conf, problem=(2, 3))
        self.check('- "multi\n'
                   '    line"\n', conf, problem=(2, 5))
        self.check('a key: "multi\n'
                   '  line"\n', conf, problem=(2, 3))
        self.check('a key: "multi\n'
                   '       line"\n', conf, problem=(2, 8))
        self.check('a key: "multi\n'
                   '         line"\n', conf, problem=(2, 10))
        self.check('a key:\n'
                   '  "multi\n'
                   '  line"\n', conf, problem=(3, 3))
        self.check('a key:\n'
                   '  "multi\n'
                   '    line"\n', conf, problem=(3, 5))
        self.check('- jinja2: "{% if ansible is defined %}\n'
                   '             {{ ansible }}\n'
                   '           {% else %}\n'
                   '             {{ chef }}\n'
                   '           {% endif %}"\n', conf,
                   problem1=(2, 14), problem2=(4, 14))
        self.check('- jinja2:\n'
                   '    "{% if ansible is defined %}\n'
                   '       {{ ansible }}\n'
                   '     {% else %}\n'
                   '       {{ chef }}\n'
                   '     {% endif %}"\n', conf,
                   problem1=(3, 8), problem2=(5, 8))
        self.check('["this is a very long line\n'
                   '  that needs to be split",\n'
                   ' "other line"]\n', conf)
        self.check('["this is a very long line\n'
                   ' that needs to be split",\n'
                   ' "other line"]\n', conf, problem=(2, 2))
        self.check('["this is a very long line\n'
                   '   that needs to be split",\n'
                   ' "other line"]\n', conf, problem=(2, 4))
        self.check('["multi\n'
                   '  line 1", "multi\n'
                   '            line 2"]\n', conf)
        self.check('["multi\n'
                   '  line 1", "multi\n'
                   '           line 2"]\n', conf, problem=(3, 12))
        self.check('["multi\n'
                   '  line 1", "multi\n'
                   '             line 2"]\n', conf, problem=(3, 14))

    def test_basics_folded_style(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: false}\n'
                'document-start: disable\n')
        self.check('>\n'
                   '  multi\n'
                   '  line\n', conf)
        self.check('- >\n'
                   '    multi\n'
                   '    line\n', conf)
        self.check('- key: >\n'
                   '    multi\n'
                   '    line\n', conf)
        self.check('- key:\n'
                   '    >\n'
                   '      multi\n'
                   '      line\n', conf)
        self.check('- ? >\n'
                   '      multi-line\n'
                   '      key\n'
                   '  : >\n'
                   '      multi-line\n'
                   '      value\n', conf)
        self.check('- ?\n'
                   '    >\n'
                   '      multi-line\n'
                   '      key\n'
                   '  :\n'
                   '    >\n'
                   '      multi-line\n'
                   '      value\n', conf)
        self.check('- jinja2: >\n'
                   '    {% if ansible is defined %}\n'
                   '      {{ ansible }}\n'
                   '    {% else %}\n'
                   '      {{ chef }}\n'
                   '    {% endif %}\n', conf)

    def test_check_multi_line_folded_style(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('>\n'
                   '  multi\n'
                   '   line\n', conf, problem=(3, 4))
        self.check('- >\n'
                   '    multi\n'
                   '     line\n', conf, problem=(3, 6))
        self.check('- key: >\n'
                   '    multi\n'
                   '     line\n', conf, problem=(3, 6))
        self.check('- key:\n'
                   '    >\n'
                   '      multi\n'
                   '       line\n', conf, problem=(4, 8))
        self.check('- ? >\n'
                   '      multi-line\n'
                   '       key\n'
                   '  : >\n'
                   '      multi-line\n'
                   '       value\n', conf,
                   problem1=(3, 8), problem2=(6, 8))
        self.check('- ?\n'
                   '    >\n'
                   '      multi-line\n'
                   '       key\n'
                   '  :\n'
                   '    >\n'
                   '      multi-line\n'
                   '       value\n', conf,
                   problem1=(4, 8), problem2=(8, 8))
        self.check('- jinja2: >\n'
                   '    {% if ansible is defined %}\n'
                   '      {{ ansible }}\n'
                   '    {% else %}\n'
                   '      {{ chef }}\n'
                   '    {% endif %}\n', conf,
                   problem1=(3, 7), problem2=(5, 7))

    def test_basics_literal_style(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: false}\n'
                'document-start: disable\n')
        self.check('|\n'
                   '  multi\n'
                   '  line\n', conf)
        self.check('- |\n'
                   '    multi\n'
                   '    line\n', conf)
        self.check('- key: |\n'
                   '    multi\n'
                   '    line\n', conf)
        self.check('- key:\n'
                   '    |\n'
                   '      multi\n'
                   '      line\n', conf)
        self.check('- ? |\n'
                   '      multi-line\n'
                   '      key\n'
                   '  : |\n'
                   '      multi-line\n'
                   '      value\n', conf)
        self.check('- ?\n'
                   '    |\n'
                   '      multi-line\n'
                   '      key\n'
                   '  :\n'
                   '    |\n'
                   '      multi-line\n'
                   '      value\n', conf)
        self.check('- jinja2: |\n'
                   '    {% if ansible is defined %}\n'
                   '     {{ ansible }}\n'
                   '    {% else %}\n'
                   '      {{ chef }}\n'
                   '    {% endif %}\n', conf)

    def test_check_multi_line_literal_style(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('|\n'
                   '  multi\n'
                   '   line\n', conf, problem=(3, 4))
        self.check('- |\n'
                   '    multi\n'
                   '     line\n', conf, problem=(3, 6))
        self.check('- key: |\n'
                   '    multi\n'
                   '     line\n', conf, problem=(3, 6))
        self.check('- key:\n'
                   '    |\n'
                   '      multi\n'
                   '       line\n', conf, problem=(4, 8))
        self.check('- ? |\n'
                   '      multi-line\n'
                   '       key\n'
                   '  : |\n'
                   '      multi-line\n'
                   '       value\n', conf,
                   problem1=(3, 8), problem2=(6, 8))
        self.check('- ?\n'
                   '    |\n'
                   '      multi-line\n'
                   '       key\n'
                   '  :\n'
                   '    |\n'
                   '      multi-line\n'
                   '       value\n', conf,
                   problem1=(4, 8), problem2=(8, 8))
        self.check('- jinja2: |\n'
                   '    {% if ansible is defined %}\n'
                   '      {{ ansible }}\n'
                   '    {% else %}\n'
                   '      {{ chef }}\n'
                   '    {% endif %}\n', conf,
                   problem1=(3, 7), problem2=(5, 7))

    # The following "paragraph" examples are inspired from
    # http://stackoverflow.com/questions/3790454/in-yaml-how-do-i-break-a-string-over-multiple-lines

    def test_paragraph_plain(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('- long text: very "long"\n'
                   '             \'string\' with\n'
                   '\n'
                   '             paragraph gap, \\n and\n'
                   '             spaces.\n', conf)
        self.check('- long text: very "long"\n'
                   '    \'string\' with\n'
                   '\n'
                   '    paragraph gap, \\n and\n'
                   '    spaces.\n', conf,
                   problem1=(2, 5), problem2=(4, 5), problem3=(5, 5))
        self.check('- long text:\n'
                   '    very "long"\n'
                   '    \'string\' with\n'
                   '\n'
                   '    paragraph gap, \\n and\n'
                   '    spaces.\n', conf)

    def test_paragraph_double_quoted(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('- long text: "very \\"long\\"\n'
                   '              \'string\' with\n'
                   '\n'
                   '              paragraph gap, \\n and\n'
                   '              spaces."\n', conf)
        self.check('- long text: "very \\"long\\"\n'
                   '    \'string\' with\n'
                   '\n'
                   '    paragraph gap, \\n and\n'
                   '    spaces."\n', conf,
                   problem1=(2, 5), problem2=(4, 5), problem3=(5, 5))
        self.check('- long text: "very \\"long\\"\n'
                   '\'string\' with\n'
                   '\n'
                   'paragraph gap, \\n and\n'
                   'spaces."\n', conf,
                   problem1=(2, 1), problem2=(4, 1), problem3=(5, 1))
        self.check('- long text:\n'
                   '    "very \\"long\\"\n'
                   '     \'string\' with\n'
                   '\n'
                   '     paragraph gap, \\n and\n'
                   '     spaces."\n', conf)

    def test_paragraph_single_quoted(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('- long text: \'very "long"\n'
                   '              \'\'string\'\' with\n'
                   '\n'
                   '              paragraph gap, \\n and\n'
                   '              spaces.\'\n', conf)
        self.check('- long text: \'very "long"\n'
                   '    \'\'string\'\' with\n'
                   '\n'
                   '    paragraph gap, \\n and\n'
                   '    spaces.\'\n', conf,
                   problem1=(2, 5), problem2=(4, 5), problem3=(5, 5))
        self.check('- long text: \'very "long"\n'
                   '\'\'string\'\' with\n'
                   '\n'
                   'paragraph gap, \\n and\n'
                   'spaces.\'\n', conf,
                   problem1=(2, 1), problem2=(4, 1), problem3=(5, 1))
        self.check('- long text:\n'
                   '    \'very "long"\n'
                   '     \'\'string\'\' with\n'
                   '\n'
                   '     paragraph gap, \\n and\n'
                   '     spaces.\'\n', conf)

    def test_paragraph_folded(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('- long text: >\n'
                   '    very "long"\n'
                   '    \'string\' with\n'
                   '\n'
                   '    paragraph gap, \\n and\n'
                   '    spaces.\n', conf)
        self.check('- long text: >\n'
                   '    very "long"\n'
                   '     \'string\' with\n'
                   '\n'
                   '      paragraph gap, \\n and\n'
                   '       spaces.\n', conf,
                   problem1=(3, 6), problem2=(5, 7), problem3=(6, 8))

    def test_paragraph_literal(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('- long text: |\n'
                   '    very "long"\n'
                   '    \'string\' with\n'
                   '\n'
                   '    paragraph gap, \\n and\n'
                   '    spaces.\n', conf)
        self.check('- long text: |\n'
                   '    very "long"\n'
                   '     \'string\' with\n'
                   '\n'
                   '      paragraph gap, \\n and\n'
                   '       spaces.\n', conf,
                   problem1=(3, 6), problem2=(5, 7), problem3=(6, 8))

    def test_consistent(self):
        conf = ('indentation: {spaces: consistent,\n'
                '              check-multi-line-strings: true}\n'
                'document-start: disable\n')
        self.check('multi\n'
                   'line\n', conf)
        self.check('multi\n'
                   ' line\n', conf, problem=(2, 2))
        self.check('- multi\n'
                   '  line\n', conf)
        self.check('- multi\n'
                   '   line\n', conf, problem=(2, 4))
        self.check('a key: multi\n'
                   '  line\n', conf, problem=(2, 3))
        self.check('a key: multi\n'
                   '        line\n', conf, problem=(2, 9))
        self.check('a key:\n'
                   '  multi\n'
                   '   line\n', conf, problem=(3, 4))
        self.check('- C code: void main() {\n'
                   '              printf("foo");\n'
                   '          }\n', conf, problem=(2, 15))
        self.check('- C code:\n'
                   '    void main() {\n'
                   '        printf("foo");\n'
                   '    }\n', conf, problem=(3, 9))
        self.check('>\n'
                   '  multi\n'
                   '  line\n', conf)
        self.check('>\n'
                   '     multi\n'
                   '     line\n', conf)
        self.check('>\n'
                   '     multi\n'
                   '      line\n', conf, problem=(3, 7))
